#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestCreateEmptyEventTreeForStudy(unittest.TestCase):

    def setUp(self):
        self.all_form_events = """
<all_form_events>
    <form>
        <name>cbc</name>
        <event>
            <name>1_arm_1</name>
            <field><name>lymce_lborres</name><value/></field><field><name>lymce_lborresu</name><value/></field><field><name>hemo_lborresu</name><value/></field><field><name>cbc_lbdtc</name><value/></field><field><name>cbc_nximport</name><value/></field><field><name>lymce_lbstat</name><value/></field><field><name>cbc_complete</name><value/></field><field><name>hemo_lbstat</name><value/></field><field><name>hemo_lborres</name><value/>
            </field>
        </event>
    </form>
    <form>
        <name>chemistry</name>
        <event>
            <name>1_arm_1</name>
            <field><name>k_lborres</name><value/></field><field><name>chem_lbdtc</name><value/></field><field><name>sodium_lborresu</name><value/></field><field><name>k_lbstat</name><value/></field><field><name>sodium_lbstat</name><value/></field><field><name>chem_nximport</name><value/></field><field><name>chemistry_complete</name><value/></field><field><name>k_lborresu</name><value/></field><field><name>sodium_lborres</name><value/>
            </field>
        </event>
    </form>
    <form>
        <name>inr</name>
            <event>
                <name>1_arm_1</name>
                <field><name>inr_lbdtc</name><value/></field><field><name>inr_complete</name><value/></field><field><name>inr_nximport</name><value/></field>
            </event>
    </form>
</all_form_events>
        """
        self.data_all_form_events= etree.ElementTree(etree.fromstring(self.all_form_events))
        return()

    def test_create_empty_event_tree_for_study_for_zero_subjects(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.zero_subjects = """<?xml version='1.0' encoding='US-ASCII'?>
        <study></study>
        """
        self.data_zero_subjects = etree.ElementTree(etree.fromstring(self.zero_subjects))
        self.assertRaises(Exception,redi.create_empty_event_tree_for_study,self.data_zero_subjects,self.data_all_form_events)
        
    def test_create_empty_event_tree_for_study_for_one_subjects(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.one_subject = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject lab_id="999-123">
            <NAME>TestSubject</NAME>
            <loinc_code>123456</loinc_code>
            <RESULT>123</RESULT>
            <REFERENCE_LOW>12.3</REFERENCE_LOW>
            <REFERENCE_HIGH>12.3</REFERENCE_HIGH>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <DATE_TIME_STAMP/>
            <STUDY_ID>123</STUDY_ID>
        <timestamp/><redcapFormName>cbc</redcapFormName><eventName/><formDateField>cbc_lbdtc</formDateField><formCompletedFieldName>cbc_complete</formCompletedFieldName><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>
        </study>
            """
        self.data_one_subject= etree.ElementTree(etree.fromstring(self.one_subject))
        
        self.output_one_subject = """
<person_form_event>
    <person lab_id="999-123">
        <study_id>123</study_id>
        <all_form_events>
        <form>
            <name>cbc</name>
            <event>
                <name>1_arm_1</name>
                <field><name>lymce_lborres</name><value/></field><field><name>lymce_lborresu</name><value/></field><field><name>hemo_lborresu</name><value/></field><field><name>cbc_lbdtc</name><value/></field><field><name>cbc_nximport</name><value/></field><field><name>lymce_lbstat</name><value/></field><field><name>cbc_complete</name><value/></field><field><name>hemo_lbstat</name><value/></field><field><name>hemo_lborres</name><value/></field>
            </event>
            </form>
        <form>
            <name>chemistry</name>
            <event>
                <name>1_arm_1</name>
                <field><name>k_lborres</name><value/></field><field><name>chem_lbdtc</name><value/></field><field><name>sodium_lborresu</name><value/></field><field><name>k_lbstat</name><value/></field><field><name>sodium_lbstat</name><value/></field><field><name>chem_nximport</name><value/></field><field><name>chemistry_complete</name><value/></field><field><name>k_lborresu</name><value/></field><field><name>sodium_lborres</name><value/></field>
                </event>
        </form>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field><field><name>inr_complete</name><value/></field><field><name>inr_nximport</name><value/></field></event>
           </form>
        </all_form_events>
    </person>
</person_form_event>
        """
        self.expect_one_subject = etree.tostring(etree.fromstring(self.output_one_subject))
        self.result = etree.tostring(
            redi.create_empty_event_tree_for_study(self.data_one_subject,self.data_all_form_events))
        clean_expected = ''.join(self.expect_one_subject.split())
        clean_result = ''.join(self.result.split())
        self.assertEqual(clean_expected, clean_result)

    def test_create_empty_event_tree_for_study_for_two_subjects(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.two_subjects = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
                <subject lab_id="999-123">
                <NAME>TestSubject_1</NAME>
                <loinc_code>123456</loinc_code>
                <RESULT>123</RESULT>
                <REFERENCE_LOW>12.3</REFERENCE_LOW>
                <REFERENCE_HIGH>12.3</REFERENCE_HIGH>
                <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
                <DATE_TIME_STAMP/>
                <STUDY_ID>123</STUDY_ID>
            <timestamp/><redcapFormName>cbc</redcapFormName><eventName/><formDateField>cbc_lbdtc</formDateField><formCompletedFieldName>cbc_complete</formCompletedFieldName><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>
            <subject lab_id="999-1234">
            <NAME>TestSubject_2</NAME>
            <loinc_code>123456</loinc_code>
            <RESULT>123</RESULT>
            <REFERENCE_LOW>12.3</REFERENCE_LOW>
            <REFERENCE_HIGH>12.3</REFERENCE_HIGH>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <DATE_TIME_STAMP/>
            <STUDY_ID>1234</STUDY_ID>
        <timestamp/><redcapFormName>cbc</redcapFormName><eventName/><formDateField>cbc_lbdtc</formDateField><formCompletedFieldName>cbc_complete</formCompletedFieldName><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>
        </study>
            """
        self.data_two_subjects= etree.ElementTree(etree.fromstring(self.two_subjects))
        
        self.output_two_subjects = """
<person_form_event>
    <person lab_id="999-1234">
        <study_id>1234</study_id>
        <all_form_events>
        <form>
            <name>cbc</name>
            <event>
                <name>1_arm_1</name>
                <field><name>lymce_lborres</name><value/></field><field><name>lymce_lborresu</name><value/></field><field><name>hemo_lborresu</name><value/></field><field><name>cbc_lbdtc</name><value/></field><field><name>cbc_nximport</name><value/></field><field><name>lymce_lbstat</name><value/></field><field><name>cbc_complete</name><value/></field><field><name>hemo_lbstat</name><value/></field><field><name>hemo_lborres</name><value/></field>
            </event>
        </form>
        <form>
            <name>chemistry</name>
            <event>
                <name>1_arm_1</name>
                <field><name>k_lborres</name><value/></field><field><name>chem_lbdtc</name><value/></field><field><name>sodium_lborresu</name><value/></field><field><name>k_lbstat</name><value/></field><field><name>sodium_lbstat</name><value/></field><field><name>chem_nximport</name><value/></field><field><name>chemistry_complete</name><value/></field><field><name>k_lborresu</name><value/></field><field><name>sodium_lborres</name><value/></field>
            </event>
        </form>
        <form>
            <name>inr</name>
            <event>
                <name>1_arm_1</name>
                <field><name>inr_lbdtc</name><value/></field><field><name>inr_complete</name><value/></field><field><name>inr_nximport</name><value/></field>
            </event>
        </form>
        </all_form_events>
    </person>
    <person lab_id="999-123">
        <study_id>123</study_id>
        <all_form_events>
        <form>
            <name>cbc</name>
            <event>
                <name>1_arm_1</name>
                <field><name>lymce_lborres</name><value/></field><field><name>lymce_lborresu</name><value/></field><field><name>hemo_lborresu</name><value/></field><field><name>cbc_lbdtc</name><value/></field><field><name>cbc_nximport</name><value/></field><field><name>lymce_lbstat</name><value/></field><field><name>cbc_complete</name><value/></field><field><name>hemo_lbstat</name><value/></field><field><name>hemo_lborres</name><value/></field>
            </event>
        </form>
        <form>
            <name>chemistry</name>
            <event>
                <name>1_arm_1</name>
                <field><name>k_lborres</name><value/></field><field><name>chem_lbdtc</name><value/></field><field><name>sodium_lborresu</name><value/></field><field><name>k_lbstat</name><value/></field><field><name>sodium_lbstat</name><value/></field><field><name>chem_nximport</name><value/></field><field><name>chemistry_complete</name><value/></field><field><name>k_lborresu</name><value/></field><field><name>sodium_lborres</name><value/></field>
            </event>
        </form>
        <form>
            <name>inr</name>
            <event>
                <name>1_arm_1</name>
                <field><name>inr_lbdtc</name><value/></field><field><name>inr_complete</name><value/></field><field><name>inr_nximport</name><value/></field>
            </event>
        </form>
        </all_form_events>
    </person>
</person_form_event>
        """
        self.expect_two_subjects = etree.tostring(etree.fromstring(self.output_two_subjects))
        self.result = etree.tostring(
            redi.create_empty_event_tree_for_study(self.data_two_subjects,self.data_all_form_events))
        clean_expected = ''.join(self.expect_two_subjects.split())
        clean_result = ''.join(self.result.split())
        self.assertEqual(clean_expected, clean_result)

    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

"""
Functions related to uploading data to REDCap
"""

import ast
import collections
import datetime
import logging
import os

from lxml import etree
from redcap import RedcapError

from utils import throttle

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def create_import_data_json(import_data_dict, event_tree):
    """
    Convert data from event_tree to json format.

    @TODO: evaluate performance
    @see the caller {@link #redi.upload.generate_output()}

    :param: import_data_dict: holds the event tree data
    :param: event_tree: holds the event tree data
    :rtype: dict
    :return the json version of the xml data
    """

    root = event_tree

    event_name = root.find('name')
    if event_name is None or not event_name.text:
        raise Exception('Expected non-blank element event/name')

    import_data_dict['redcap_event_name'] = event_name.text

    event_field_value_list = root.xpath('//event/field/name')

    for name in event_field_value_list:
        if name.text is None:
            raise Exception(
                'Expected non-blank element event/field/name')

    # Match all fields to build a row for each
    event_field_list = root.xpath('field')
    contains_data = False

    for field in event_field_list:
        val = field.findtext('value', '')
        import_data_dict[field.findtext('name')] = val

        if val and not contains_data:
            contains_data = True

    return {'json_data': import_data_dict, 'contains_data': contains_data}


def create_redcap_records(import_data):
    """
    Creates REDCap records from RED-I's form data, AKA import data.

    REDCap API only accepts records for importing. Records are differentiated by
    their unique record ID, unless the REDCap Project is a Longitudinal study.
    In that case, they are differentiated by a combination of record ID and an
    event.

    Since RED-I views the world in terms of forms, we have to project our
    form-centric view into REDCap's record-centric world. This is done by
    combining all form data with the same Subject ID and Event Name into the
    same record.

    :param import_data: iterable of 4-tuples: (study_id_key, form_name,
        event_name, json_data_dict)
    :return: iterable of REDCap records ready for upload
    """
    records_by_subject_and_event = collections.defaultdict(dict)
    for subject_id_key, _, event_name, record in import_data:
        records_by_subject_and_event[subject_id_key, event_name].update(record)
    return records_by_subject_and_event.itervalues()


def generate_output(person_tree, redcap_client, rate_limit, sent_events,
    max_retry_count, skip_blanks=False, bulk_send_blanks=False):
    """
    Note: This function communicates with the redcap application.
    Steps:
        - loop for each person/form/event element
        - generate a csv fragment `using create_eav_output`
        - send csv fragment to REDCap using `send_eav_data_to_redcap`

    @see the caller {@link #redi.redi._run()}

    :rtype:     dictionary
    :return:    the report_data which is passed to the report rendering function
    """

    # the global dictionary to be returned
    report_data = {
        'errors': []
    }

    """
     For each person we keep a count for each form type:
        subject_details = array(
            'person_A' => array('form_1': 1, 'form_2': 10, ...
            'person_B' => array('form_1': 1, 'form_2': 10, ...
            ...
        )
    """
    subject_details = {}

    # For each form type we keep a global count
    form_details = {}

    # count how many `person` elements are parsed
    person_count = 0

    root = person_tree.getroot()
    persons = root.xpath('//person')

    upload_data = throttle.Throttle(redcap_client.send_data_to_redcap,
                                    int(rate_limit))

    blanks = []

    # main loop for each person
    for person in persons:
        time_begin = datetime.datetime.now()
        person_count += 1
        study_id = (person.xpath('study_id') or [None])[0]

        if study_id is None:
            raise Exception('Expected a valid value for study_id')

        # count how many csv fragments are created per person
        event_count = 0
        logger.info('Start sending data for study_id: %s' % study_id.text)

        forms = person.xpath('./all_form_events/form')

        # loop through the forms of one person
        for form in forms:
            form_name = form.xpath('name')[0].text
            form_key = 'Total_' + form_name + '_Forms'
            study_id_key = study_id.text

            # init dictionary for a new person in (study_id)
            if study_id_key not in subject_details:
                subject_details[study_id_key] = {}
                subject_details[study_id_key]['lab_id'] = person.get('lab_id')

            if not form_key in subject_details[study_id_key]:
                subject_details[study_id_key][form_key] = 0

            if form_key not in form_details:
                form_details[form_key] = 0

            logger.debug(
                'parsing study_id ' +
                study_id.text +
                ' form: ' +
                form_name)

            # loop through the events of one form
            for event in form.xpath('event'):
                event_name = event.findtext('name', '')
                assert event_name, "Missing name for form event"

                try:
                    import_dict = {
                        redcap_client.project.def_field: study_id.text}
                    import_dict = create_import_data_json(
                        import_dict,
                        event)
                    json_data_dict = import_dict['json_data']
                    contains_data = import_dict['contains_data']

                    if sent_events.was_sent(study_id_key, form_name, event_name):
                        logger.debug("Skipping previously sent " + event_name)
                        if contains_data:
                            # if no error_strings encountered update event counters
                            subject_details[study_id_key][form_key] += 1
                            form_details[form_key] += 1
                        continue

                    is_blank = not contains_data
                    if is_blank:
                        if skip_blanks:
                            # assume subsequent events for this form and subject
                            # are blank and simply move on to the next form by
                            # breaking out of the events-loop
                            break

                        if bulk_send_blanks:
                            blanks.append((study_id_key, form_name, event_name,
                                           json_data_dict))
                            continue

                    event_count += 1
                    if (0 == event_count % 50):
                        logger.info('Requests sent: %s' % (event_count))

                    # to speedup testing uncomment the following line
                    # if (0 == event_count % 2) : continue

                    try:
                        upload_data([json_data_dict], max_retry_count,
                            overwrite=True)
                        sent_events.mark_sent(study_id_key, form_name, event_name)
                        logger.debug("Sent " + event_name)
                        if contains_data:
                            # if no errors encountered update event counters
                            subject_details[study_id_key][form_key] += 1
                            form_details[form_key] += 1
                    except RedcapError as redcap_err:
                        handle_errors_in_redcap_xml_response(study_id, redcap_err, report_data)

                except Exception as e:
                    logger.error(e.message)
                    raise

        time_end = datetime.datetime.now()
        logger.info("Total execution time for study_id %s was %s" % (study_id_key, (time_end - time_begin)))
        logger.info("Total REDCap requests sent: %s \n" % (event_count))

    if blanks:
        logger.info("Sending blank forms in bulk...")
        records = list(create_redcap_records(blanks))

        try:
            response = upload_data(records, overwrite=True)
            for study_id_key, form_name, event_name, record in blanks:
                sent_events.mark_sent(study_id_key, form_name, event_name)
            logger.info("Sent {} blank form-events.".format(response['count']))

        except RedcapError as redcap_err:
            logger.error("Failed to send blank form-events.")
            handle_errors_in_redcap_xml_response(study_id, redcap_err, report_data)

    report_data.update({
        'total_subjects': person_count,
        'form_details': form_details,
        'subject_details': subject_details,
        'errors': report_data['errors']
    })

    logger.debug('report_data ' + repr(report_data))
    return report_data


def handle_errors_in_redcap_xml_response(study_id, redcap_err, report_data):
    """
    Checks for any errors in the redcap response and update
    report data if there are any errors.

    Parameters:
    -----------
    redcap_err: RedcapError object
    report_data: dictionary to which we store error details
    """

    # converting string to dictionary
    response = ast.literal_eval(str(redcap_err))
    logger.debug('handling response from the REDCap')

    if 'error' not in response:
        logger.warn("RedcapError does not contain the expected 'error' key: {}"
                .format(response))
        return

    if 'records' in response:
        records = response['records']

        for record in records:
            details = "(record: {}, field_name: {}, value: {}, message: {})" \
                    .format(
                            record['record'],
                            record['field_name'],
                            record['value'],
                            record['message'])
            error_string = "{}: {}".format(response['error'], details)
            report_data['errors'].append(error_string)
            logger.error("{}".format(error_string))

    elif 'fields' in response:
        fields = response['fields']
        details = "{}".format(fields)
        error_string = "{}: {}".format(response['error'], details)
        report_data['errors'].append(error_string)
        logger.error(error_string)

    else:
        err = "A RedcapError ocured for study_id: {}. " \
                "It contains an unexpected type of error: {}" \
                .format(study_id, response)
        logger.warn(err)



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

'''
@author : Ruchi
email : ruchi.desai@ufl.edu

This file is to test the function send_data_to_redcap

'''
import unittest
from mock import patch, call
from redi import redi
from redi.utils import redcapClient

class TestSendDatatoRedcap(unittest.TestCase):

    def setUp(self):
        self.test_data = ''

    def dummy_init(*args, **kwargs):
        """This function is called in place of RedcapClient's constructor"""
        return None

    def dummy_send_data_to_redcap(*args, **kwargs):
        """No time out has occurred or function retries to send the data on 
        network connection timeout"""
        return True

    def dummy_send_data_to_redcap_timeout(*args, **kwargs):
        """retry_count has reached 10 so the function stops resending data 
        and exits gracefully"""
        return True

    @patch.multiple(redcapClient.RedcapClient, __init__=dummy_init)
    @patch.multiple(redcapClient.RedcapClient,
        send_data_to_redcap=dummy_send_data_to_redcap)
    def test_send_data_to_redcap(self):
        self.assertTrue(redcapClient.RedcapClient().send_data_to_redcap(
            self.test_data, False, 0))

    @patch.multiple(redcapClient.RedcapClient, __init__=dummy_init)
    @patch.multiple(redcapClient.RedcapClient,
        send_data_to_redcap=dummy_send_data_to_redcap)
    def test_send_data_to_redcap_retry_3(self):       
        self.assertTrue(redcapClient.RedcapClient().send_data_to_redcap(
            self.test_data, 'overwrite', 3))

    @patch.multiple(redcapClient.RedcapClient, __init__=dummy_init)
    @patch.multiple(redcapClient.RedcapClient,
        send_data_to_redcap=dummy_send_data_to_redcap_timeout)
    def test_send_data_to_redcap_retry_10(self):
        self.assertTrue(redcapClient.RedcapClient().send_data_to_redcap(
            self.test_data, 'overwrite', 10))

    def tearDown(self):
        return()

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

'''
@author : Mohan
email : mohan88@ufl.edu

This file tests for the function parse_raw_xml

'''
import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestConvertComponentIdToLoincCode(unittest.TestCase):

    def setUp(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        
        
    def test_convert_component_id_to_loinc_code(self):
        self.sampleXmlMapData = """<?xml version='1.0' encoding='US-ASCII'?>
<clinical_datum>
	<version>0.1.0</version>
	<Description>Test Description</Description>
	<components>
		<component>
			<description>Test Component 1</description>
			<source>
				<name>Comp_id</name>
				<value>123</value>
			</source>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>

		<component>
			<description>Test Component 2</description>
			<source>
				<name>Comp_id</name>
				<value>789</value>
			</source>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>
	</components>
</clinical_datum>"""
        self.mapxmlDataTree= etree.ElementTree(etree.fromstring(self.sampleXmlMapData))
        
        self.rawXmlData = """<?xml version="1.0" encoding="utf8"?>
        <study>
            <subject>
                <NAME>test subject 1</NAME>
                <Comp_id>123</Comp_id>
                <RESULT>12.3</RESULT>
                <STUDY_ID>4567</STUDY_ID>
            </subject>
            <subject>
                <NAME>test subject 2</NAME>
                <Comp_id>789</Comp_id>
                <RESULT>12.3</RESULT>
                <STUDY_ID>4537</STUDY_ID>
            </subject>
        </study>"""
        self.rawxmlDataTree= etree.ElementTree(etree.fromstring(self.rawXmlData))
        
        self.result = redi.convert_component_id_to_loinc_code(self.rawxmlDataTree,self.mapxmlDataTree)
        
        self.expect =  etree.tostring(self.result, method='html', pretty_print=True)
        self.output = """<study>
            <subject>
                <NAME>test subject 1</NAME>
                <lcode>456</lcode><RESULT>12.3</RESULT>
                <STUDY_ID>4567</STUDY_ID>
            </subject>
            <subject>
                <NAME>test subject 2</NAME>
                <lcode>456</lcode><RESULT>12.3</RESULT>
                <STUDY_ID>4537</STUDY_ID>
            </subject>
        </study>
"""
        
        self.assertEqual(self.expect, self.output)
    
    def test_convert_component_id_to_loinc_code_with_no_source(self):
        self.sampleXmlMapData = """<?xml version='1.0' encoding='US-ASCII'?>
<clinical_datum>
	<version>0.1.0</version>
	<Description>Test Description</Description>
	<components>
		<component>
			<description>Test Component 1</description>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>

		<component>
			<description>Test Component 2</description>
			<target>
				<name>lcode</name>
				<value>456</value>
			</target>
		</component>
	</components>
</clinical_datum>"""
        self.mapxmlDataTree= etree.ElementTree(etree.fromstring(self.sampleXmlMapData))
    
        self.rawXmlData = """<?xml version="1.0" encoding="utf8"?>
        <study>
            <subject>
                <NAME>test subject 1</NAME>
                <Comp_id>123</Comp_id>
                <RESULT>12.3</RESULT>
                <STUDY_ID>4567</STUDY_ID>
            </subject>
            <subject>
                <NAME>test subject 2</NAME>
                <Comp_id>789</Comp_id>
                <RESULT>12.3</RESULT>
                <STUDY_ID>4537</STUDY_ID>
            </subject>
        </study>"""
        self.rawxmlDataTree= etree.ElementTree(etree.fromstring(self.rawXmlData))
    
        self.assertRaises(Exception,redi.convert_component_id_to_loinc_code,self.rawxmlDataTree, self.mapxmlDataTree)
                
    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

__author__ = "Taeber Rapczak <taeber@ufl.edu>"


class Form(object):

    def __init__(self, data):
        try:
            data.xpath('.')
        except Exception as data_exception:
            print data_exception
            raise ValueError('"data" should be a valid lxml etree')

        self._tree = data

    def events(self):
        for node in self._tree.xpath("//event[.]"):
            yield Event(node)


class Event(object):

    def __init__(self, etree_node):
        try:
            etree_node.xpath('.')
        except Exception:
            print ValueError('"etree_node" should be a valid lxml node')
        self._node = etree_node

    def field(self, name):
        nodes = self._node.xpath(
            "./field[./name = $fieldName]",
            fieldName=name)
        nodecount = len(nodes)
        if nodecount == 1:
            return Field(nodes[0])
        elif nodecount == 0:
            return None
        else:
            raise Exception("Malformed XML: multiple fields with the name {0}".
                            format(name))

    def fields(self):
        for node in self._node.xpath("./field[.]"):
            yield Field(node)

    @property
    def name(self):
        return self._node.findtext('name')

    @property
    def study_id(self):
        form = self._node.getparent()
        all_form_events = form.getparent()
        person = all_form_events.getparent()
        return person.findtext('study_id', default='')

    @property
    def form_name(self):
        return self._node.getparent().findtext('name', '')

    def is_empty(self):
        return not self._node.xpath("./field/value/text() != ''")


class Field(object):

    def __init__(self, etree_node):
        try:
            etree_node.findtext('value')
        except Exception:
            print ValueError('"etree_node" should be a valid lxml node')
        self._node = etree_node

    @property
    def name(self):
        return self._node.findtext('name', default='')

    @property
    def value(self):
        return self._node.findtext('value', default='')

    @value.setter
    def value(self, value):
        self._node.find('value').text = value

    def clear_value(self):
        self.value = ''



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause
"""
SimpleConfigParser

Simple configuration file parser: Python module to parse configuration files
without sections. Based on ConfigParser from the standard library.

Author: Philippe Lagadec

Project website: http://www.decalage.info/python/configparser

Inspired from an idea posted by Fredrik Lundh:
http://mail.python.org/pipermail/python-dev/2002-November/029987.html

Usage: see end of source code and http://docs.python.org/library/configparser.html
"""

__author__ = 'Philippe Lagadec'
__version__ = '0.02'

#--- LICENSE ------------------------------------------------------------------

# The SimpleConfigParser Python module is copyright (c) Philippe Lagadec 2009-2010
#
# By obtaining, using, and/or copying this software and/or its associated
# documentation, you agree that you have read, understood, and will comply with
# the following terms and conditions:
#
# Permission to use, copy, modify, and distribute this software and its
# associated documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies, and that both
# that copyright notice and this permission notice appear in supporting
# documentation, and that the name of the author not be used in advertising or
# publicity pertaining to distribution of the software without specific,
# written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
# THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
# IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

#-------------------------------------------------------------------------
# CHANGELOG
# 2009-02-12 v0.01 PL: - initial version
# 2010-03-15 v0.02 PL: - updated tests and comments

#-------------------------------------------------------------------------
# TODO:
# - implement read() using the base class code

#=== IMPORTS ==================================================================

import ConfigParser
import StringIO
import logging
import os
import sys
logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

#=== CONSTANTS ================================================================

# section name for options without section:
NOSECTION = 'NOSECTION'

DEFAULT_MESSAGE_NO_VALUE = "Required parameter '{0}' does not have a"\
                           " value in {1}."

DEFAULT_MESSAGE = "\nPlease set it with the appropriate value. Refer to "\
                  "config-example/settings.ini for assistance.\nProgram "\
                  "will now terminate..."

# Dictionary containing required file-related parameters along with custom
# message to be displayed in case of error
required_files_dict = {
    "raw_xml_file": "\nIt should specify the name of the file containing raw"\
    " data. ",
    "translation_table_file": "\nIt should specify the name of the required "\
    "xml file containing translation table. ",
    "form_events_file": "\nIt should specify the name of the required xml "\
    "file containing empty form events. ",
    "research_id_to_redcap_id": "\nIt should specify the name of the xml "\
    "file containing mapping of research ids to redcap ids. ",
    "component_to_loinc_code_xml": "\nIt should specify the name of the "\
    "required xml file  containing a mapping of clinical component ids to "\
    "loinc codes. "
}

required_server_parameters_list = [
    'redcap_uri',
    'token',
    'redcap_support_receiver_email',
    'smtp_host_for_outbound_mail',
    'smtp_port_for_outbound_mail',
    'emr_sftp_server_hostname',
    'emr_sftp_server_username',
    'emr_sftp_project_name',
    'emr_data_file']

# Dictionary containing optional parameters along with their default values
optional_parameters_dict = {
    "report_file_path": "report.xml",
    "input_date_format": "%Y-%m-%d %H:%M:%S",
    "output_date_format": "%Y-%m-%d",
    "report_file_path2": "report.html",
    "sender_email": "please-do-not-reply@example.com",
    "project": "DEFAULT_PROJECT",
    "rules": {},
    "preprocessors": {},
    "batch_warning_days": 13,
    "rate_limiter_value_in_redcap": 600,
    "batch_info_database": "redi.db",
    "send_email": 'N',
    "receiver_email": "test@example.com",
    "verify_ssl": True,
    "replace_fields_in_raw_data_xml": None,
    "include_rule_errors_in_report": False,
    "redcap_support_sender_email": 'please-do-not-reply@example.com',
    "emr_sftp_server_port": 22,
    "emr_sftp_server_password": None,
    "emr_sftp_server_private_key": None,
    "emr_sftp_server_private_key_pass": None,
    "is_sort_by_lab_id": True,
    "max_retry_count": 10,
}

class ConfigurationError(Exception):
    pass

#=== CLASSES ==================================================================

class SimpleConfigParser(ConfigParser.RawConfigParser):

    """
    Simple configuration file parser: based on ConfigParser from the standard
    library, slightly modified to parse configuration files without sections.

    Inspired from an idea posted by Fredrik Lundh:
    http://mail.python.org/pipermail/python-dev/2002-November/029987.html
    """

    def read(self, filename):
        if not os.path.exists(filename):
            logger.exception("Cannot find settings file: {0}. Program will "\
                             "now terminate...".format(filename))
            sys.exit()

        self.filename = filename
        text = open(filename).read()
        f = StringIO.StringIO("[%s]\n" % NOSECTION + text)
        self.readfp(f, filename)

    def getoption(self, option):
        'get the value of an option'
        opt_as_string = self.get(NOSECTION, option)
        try:
            # if the conversion to boolean fails we keep the string value
            opt_as_bool = to_bool(opt_as_string)
            return opt_as_bool
        except ValueError:
            pass
        return opt_as_string

    def getoptionslist(self):
        'get a list of available options'
        return self.options(NOSECTION)

    def hasoption(self, option):
        """
        return True if an option is available, False otherwise.
        (NOTE: do not confuse with the original has_option)
        """
        return self.has_option(NOSECTION, option)

    def set_attributes(self):
        # Check if configuration file is empty
        if not self.getoptionslist():
            message = "ERROR: Configuration file '{0}' is empty! Program "\
                      "will now terminate...".format(self.filename)
            logger.error(message)
            sys.exit()

        else:
            self.check_parameters()


    def check_parameters(self):
        """
        handle required and default optional_parameters_dict
        """
        # check for required file parameters
        # handled separately as these need a custom message to be displayed
        for option in required_files_dict:
            if not self.hasoption(option) or self.getoption(option) == "":
                message = DEFAULT_MESSAGE_NO_VALUE.format(option, \
                    self.filename) + required_files_dict[option] +\
                     DEFAULT_MESSAGE
                logger.error(message)
                sys.exit()
            else:
                setattr(self, option, self.getoption(option))

        # check for required server and emr parameters
        for option in required_server_parameters_list:
            if not self.hasoption(option) or self.getoption(option) == "":
                message = DEFAULT_MESSAGE_NO_VALUE.format(option, \
                    self.filename) + DEFAULT_MESSAGE
                logger.error(message)
                sys.exit()
            else:
                logger.debug("Setting required parameter {} = {} "\
                        .format(option, self.getoption(option)))
                setattr(self, option, self.getoption(option))

        # check for receiver email if send_email = 'Y'
        if self.hasoption('send_email') and self.getoption('send_email'):
            if not self.hasoption('receiver_email') or \
            self.getoption('receiver_email') == "":
                message = DEFAULT_MESSAGE_NO_VALUE.format(option, \
                    self.filename) + DEFAULT_MESSAGE
                logger.error(message)
                sys.exit()

        # set optional parameters with default values if missing
        for option in optional_parameters_dict:
            if not self.hasoption(option) or self.getoption(option) == "":
                logger.warn("Parameter '{0}' in {1} does not have"\
                " a value. Default value '{2}' applied.".format(option, \
                    self.filename, optional_parameters_dict[option]))
                setattr(self, option, optional_parameters_dict[option])
            else:
                setattr(self, option, self.getoption(option))

#=== End class ================================================================
def to_bool(value):
    """
    Helper function for translating strings into booleans
    @see test/TestReadConfig.py
    """
    valid = {
        'true':  True,  't': True,  '1': True,  'y' : True,
        'false': False, 'f': False, '0': False, 'n' : False
        }

    if not isinstance(value, str):
        raise ValueError('Cannot check boolean value. Not a string.')

    lower_value = value.lower()
    if lower_value in valid:
        return valid[lower_value]
    else:
        raise ValueError('Not a boolean string: "%s"' % value)

#=== MAIN =====================================================================

if __name__ == '__main__':
    # simple tests when launched as a script instead of imported as module:

    ##cp = ConfigParser.ConfigParser()
    # this raises an exception:
    # ConfigParser.MissingSectionHeaderError: File contains no section headers.
    # cp.read('config_without_section.ini')

    print 'SimpleConfigParser tests:'

    filename = 'sample_config_no_section.ini'
    cp = SimpleConfigParser()
    print 'Parsing %s...' % filename
    cp.read(filename)

    print 'Sections:', cp.sections()
    # print cp.items(NOSECTION)
    print 'getoptionslist():', cp.getoptionslist()
    for option in cp.getoptionslist():
        print "getoption('%s') = '%s'" % (option, cp.getoption(option))
    print "hasoption('wrongname') =", cp.hasoption('wrongname')

    print
    print "Print out options by attribute instead of recursing the list"
    cp.set_attributes()
    print cp.option1
    print cp.option2



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import logging

from lxml import etree

from redi import redi


class TestCopyDataToPersonFormEventTree(unittest.TestCase):
    def setUp(self):
        # silence logger during testing
        redi.logger = logging.getLogger('redi')
        redi.logger.addHandler(logging.NullHandler())

        self.form_event_tree = """<?xml version='1.0' encoding='US-ASCII'?>
        <redcapProject>
        <name>Project</name>
        <form>
        <name>cbc</name>
        <formDateField>cbc_lbdtc</formDateField>
        <formCompletedFieldName>cbc_complete</formCompletedFieldName>
        <formImportedFieldName>cbc_nximport</formImportedFieldName>
        <formCompletedFieldValue>2</formCompletedFieldValue>
        <formImportedFieldValue>Y</formImportedFieldValue>
        <event>
            <name>1_arm_1</name>
        </event>


        </form>
        <form>
            <name>chemistry</name>
            <formDateField>chem_lbdtc</formDateField>
            <formCompletedFieldName>chemistry_complete</formCompletedFieldName>
            <formImportedFieldName>chem_nximport</formImportedFieldName>
            <formCompletedFieldValue>2</formCompletedFieldValue>
            <formImportedFieldValue>Y</formImportedFieldValue>
            <event>
                <name>1_arm_1</name>
            </event>


        </form>

        <form>
            <name>inr</name>
            <formDateField>inr_lbdtc</formDateField>
            <formCompletedFieldName>inr_complete</formCompletedFieldName>
            <formImportedFieldName>inr_nximport</formImportedFieldName>
            <formCompletedFieldValue>2</formCompletedFieldValue>
            <formImportedFieldValue>Y</formImportedFieldValue>
            <event>
            <name>1_arm_1</name>
            </event>

           </form>

        </redcapProject>
        """
        self.data_form_event_tree = etree.ElementTree(etree.fromstring(self.form_event_tree))

    def test_copy_data_with_blank_reference_unit(self):
        self.person_form_event_tree = """<person_form_event><person><study_id>123</study_id><all_form_events><form>
        <name>cbc</name>
        <event>
            <name>1_arm_1</name>
            <field><name>hemo_lborres</name><value/></field>
            <field><name>hemo_lborresu</name><value/></field>
            <field><name>cbc_complete</name><value/></field>
            <field><name>cbc_nximport</name><value/></field></event>
        </form>
        </all_form_events></person></person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))

        self.one_subject = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT/>
            <STUDY_ID>123</STUDY_ID>
            <timestamp>1906-12-25</timestamp>
            <redcapFormName>cbc</redcapFormName>
            <eventName>1_arm_1</eventName>
            <formDateField>cbc_lbdtc</formDateField>
            <formCompletedFieldName>cbc_complete</formCompletedFieldName>
            <formImportedFieldName>cbc_nximport</formImportedFieldName>
            <redcapFieldNameValue>hemo_lborres</redcapFieldNameValue>
            <redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits>
            <redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>
        </study>
            """
        self.data_one_subject = etree.ElementTree(etree.fromstring(self.one_subject))
        self.result = etree.tostring(
            redi.copy_data_to_person_form_event_tree(self.data_one_subject, self.data_person_form_event_tree,
                                                     self.data_form_event_tree))

        self.output = """<person_form_event><person><study_id>123</study_id><all_form_events><form>
        <name>cbc</name>
        <event>
            <name>1_arm_1</name>
            <field><name>hemo_lborres</name><value>987</value></field>
            <field><name>hemo_lborresu</name><value/></field>
            <field><name>cbc_complete</name><value>2</value></field>
            <field><name>cbc_nximport</name><value>Y</value></field></event>
        </form>
        </all_form_events></person></person_form_event>"""

        self.expect = etree.tostring(etree.fromstring(self.output))
        self.assertEqual(self.expect, self.result)

    def test_copy_data_to_person_form_event_tree_one_person(self):
        self.person_form_event_tree = """<person_form_event><person><study_id>123</study_id><all_form_events><form>
        <name>cbc</name>
        <event>
            <name>1_arm_1</name>
            <field><name>hemo_lborres</name><value/></field>
            <field><name>hemo_lborresu</name><value/></field>
            <field><name>cbc_complete</name><value/></field>
            <field><name>cbc_nximport</name><value/></field></event>
        </form>
        </all_form_events></person></person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))

        self.one_subject = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>123</STUDY_ID>
        <timestamp>1906-12-25</timestamp>
        <redcapFormName>cbc</redcapFormName>
        <eventName>1_arm_1</eventName>
        <formDateField>cbc_lbdtc</formDateField>
        <formCompletedFieldName>cbc_complete</formCompletedFieldName>
        <formImportedFieldName>cbc_nximport</formImportedFieldName>
        <redcapFieldNameValue>hemo_lborres</redcapFieldNameValue>
        <redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits>
        <redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>
        </study>
            """
        self.data_one_subject = etree.ElementTree(etree.fromstring(self.one_subject))
        self.result = etree.tostring(
            redi.copy_data_to_person_form_event_tree(self.data_one_subject, self.data_person_form_event_tree,
                                                     self.data_form_event_tree))

        self.output = """<person_form_event><person><study_id>123</study_id><all_form_events><form>
        <name>cbc</name>
        <event>
            <name>1_arm_1</name>
            <field><name>hemo_lborres</name><value>987</value></field>
            <field><name>hemo_lborresu</name><value>g/dL</value></field>
            <field><name>cbc_complete</name><value>2</value></field>
            <field><name>cbc_nximport</name><value>Y</value></field></event>
        </form>
        </all_form_events></person></person_form_event>"""

        self.expect = etree.tostring(etree.fromstring(self.output))
        self.assertEqual(self.expect, self.result)

    def test_copy_data_to_person_form_event_tree_two_persons(self):
        self.person_form_event_tree = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field>
            <field><name>inr_complete</name><value/></field>
            <field><name>inr_nximport</name><value/></field></event>

           </form>

        </all_form_events></person>
        <person><study_id>123</study_id><all_form_events><form>
                <name>cbc</name>
                <event>
                    <name>1_arm_1</name>
                    <field><name>hemo_lborres</name><value></value></field>
                    <field><name>cbc_complete</name><value/></field>
                    <field><name>cbc_nximport</name><value/></field></event>
                </form>
                </all_form_events></person>

        </person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))
        self.two_subjects = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>123</STUDY_ID>
        <timestamp>1906-12-25</timestamp><redcapFormName>cbc</redcapFormName><eventName>1_arm_1</eventName><formDateField>cbc_lbdtc</formDateField><formCompletedFieldName>cbc_complete</formCompletedFieldName><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>123</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-25</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formDateField>inr_lbdtc</formDateField><formCompletedFieldName>inr_complete</formCompletedFieldName><formImportedFieldName>inr_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_two_subjects = etree.ElementTree(etree.fromstring(self.two_subjects))
        self.result = etree.tostring(
            redi.copy_data_to_person_form_event_tree(self.data_two_subjects, self.data_person_form_event_tree,
                                                     self.data_form_event_tree))
        self.output = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value>1906-12-25</value></field>
            <field><name>inr_complete</name><value>2</value></field>
            <field><name>inr_nximport</name><value>Y</value></field></event>

           </form>

        </all_form_events></person>
        <person><study_id>123</study_id><all_form_events><form>
                <name>cbc</name>
                <event>
                    <name>1_arm_1</name>
                    <field><name>hemo_lborres</name><value>987</value></field>
                    <field><name>cbc_complete</name><value>2</value></field>
                    <field><name>cbc_nximport</name><value>Y</value></field></event>
                </form>
                </all_form_events></person>

        </person_form_event>"""

        self.expect = etree.tostring(etree.fromstring(self.output))
        self.assertEqual(self.expect, self.result)

    def test_copy_data_to_person_form_event_tree_zero_forms(self):
        self.person_form_event_tree = """<person_form_event><person><study_id>456</study_id>
        </person>

        </person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))
        self.zero_form = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>123</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-25</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formDateField>inr_lbdtc</formDateField><formCompletedFieldName>inr_complete</formCompletedFieldName><formImportedFieldName>inr_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_zero_form = etree.ElementTree(etree.fromstring(self.zero_form))
        self.assertRaises(Exception, redi.copy_data_to_person_form_event_tree, self.data_zero_form,
                          self.data_person_form_event_tree, self.data_form_event_tree)

    def test_copy_data_to_person_form_event_tree_two_forms(self):
        self.person_form_event_tree = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>cbc</name>
            <event>
            <name>1_arm_1</name>
            <field><name>cbc_lbdtc</name><value/></field>
            <field><name>cbc_complete</name><value/></field>
            <field><name>cbc_nximport</name><value/></field></event>
            </form>
            <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field>
            <field><name>inr_complete</name><value/></field>
            <field><name>inr_nximport</name><value/></field>
            </event>
            </form>
        </all_form_events></person></person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))
        self.two_subjects = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-26</timestamp><redcapFormName>cbc</redcapFormName><eventName>1_arm_1</eventName><formDateField>cbc_lbdtc</formDateField><formCompletedFieldName>cbc_complete</formCompletedFieldName><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>123</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-25</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formDateField>inr_lbdtc</formDateField><formCompletedFieldName>inr_complete</formCompletedFieldName><formImportedFieldName>inr_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_two_subjects = etree.ElementTree(etree.fromstring(self.two_subjects))
        self.result = etree.tostring(
            redi.copy_data_to_person_form_event_tree(self.data_two_subjects, self.data_person_form_event_tree,
                                                     self.data_form_event_tree))
        self.output = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>cbc</name>
            <event>
            <name>1_arm_1</name>
            <field><name>cbc_lbdtc</name><value>1906-12-26</value></field>
            <field><name>cbc_complete</name><value>2</value></field>
            <field><name>cbc_nximport</name><value>Y</value></field></event>
            </form>
            <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value>1906-12-25</value></field>
            <field><name>inr_complete</name><value>2</value></field>
            <field><name>inr_nximport</name><value>Y</value></field>
            </event>
            </form>
        </all_form_events></person></person_form_event>"""

        self.expect = etree.tostring(etree.fromstring(self.output))
        self.assertEqual(self.expect, self.result)

    def test_copy_data_to_person_form_event_tree_two_events(self):
        self.person_form_event_tree = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field>
            <field><name>inr_complete</name><value/></field>
            <field><name>inr_nximport</name><value/></field>
            </event>
            <event>
            <name>2_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field>
            <field><name>inr_complete</name><value/></field>
            <field><name>inr_nximport</name><value/></field>
            </event>
           </form>
        </all_form_events></person></person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))
        self.two_subjects = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-26</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formDateField>inr_lbdtc</formDateField><formCompletedFieldName>inr_complete</formCompletedFieldName><formImportedFieldName>inr_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>123</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-25</timestamp><redcapFormName>inr</redcapFormName><eventName>2_arm_1</eventName><formDateField>inr_lbdtc</formDateField><formCompletedFieldName>inr_complete</formCompletedFieldName><formImportedFieldName>inr_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_two_subjects = etree.ElementTree(etree.fromstring(self.two_subjects))
        # print "Testing Test case"
        self.result = etree.tostring(
            redi.copy_data_to_person_form_event_tree(self.data_two_subjects, self.data_person_form_event_tree,
                                                     self.data_form_event_tree))
        self.output = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value>1906-12-26</value></field>
            <field><name>inr_complete</name><value>2</value></field>
            <field><name>inr_nximport</name><value>Y</value></field>
            </event>
            <event>
            <name>2_arm_1</name>
            <field><name>inr_lbdtc</name><value>1906-12-25</value></field>
            <field><name>inr_complete</name><value>2</value></field>
            <field><name>inr_nximport</name><value>Y</value></field>
            </event>
           </form>
        </all_form_events></person></person_form_event>"""

        self.expect = etree.tostring(etree.fromstring(self.output))
        self.assertEqual(self.expect, self.result)

    def test_form_date_field_pair(self):
        self.person_form_event_tree = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field></event>
        </form>
        </all_form_events></person></person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))
        self.form_date_field = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-26</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formCompletedFieldName>cbc_complete</formCompletedFieldName><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_form_date_field = etree.ElementTree(etree.fromstring(self.form_date_field))
        self.form_date_value_field = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formCompletedFieldName>cbc_complete</formCompletedFieldName><formDateField>cbc_lbdtc</formDateField><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_form_date_value_field = etree.ElementTree(etree.fromstring(self.form_date_value_field))

        self.assertRaises(Exception, redi.copy_data_to_person_form_event_tree, self.data_form_date_field,
                          self.data_person_form_event_tree, self.data_form_event_tree)
        self.assertRaises(Exception, redi.copy_data_to_person_form_event_tree, self.data_form_date_value_field,
                          self.data_person_form_event_tree, self.data_form_event_tree)

    def test_redcap_field_name_pair(self):
        self.person_form_event_tree = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field></event>
        </form>
        </all_form_events></person></person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))
        self.redcap_field_name = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-26</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formCompletedFieldName>cbc_complete</formCompletedFieldName><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_redcap_field_name = etree.ElementTree(etree.fromstring(self.redcap_field_name))
        self.redcap_value_field = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formCompletedFieldName>cbc_complete</formCompletedFieldName><formDateField>cbc_lbdtc</formDateField><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_redcap_value_field = etree.ElementTree(etree.fromstring(self.redcap_value_field))
        self.assertRaises(Exception, redi.copy_data_to_person_form_event_tree, self.data_redcap_field_name,
                          self.data_person_form_event_tree, self.data_form_event_tree)
        self.assertRaises(Exception, redi.copy_data_to_person_form_event_tree, self.data_redcap_value_field,
                          self.data_person_form_event_tree, self.data_form_event_tree)

    def test_redcap_field_units_pair(self):
        self.person_form_event_tree = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field></event>
        </form>
        </all_form_events></person></person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))
        self.redcap_field_units_name = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-26</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formDateField>cbc_lbdtc</formDateField><formCompletedFieldName>cbc_complete</formCompletedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_redcap_field_units_name = etree.ElementTree(etree.fromstring(self.redcap_field_units_name))
        self.redcap_units_value_field = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>987</RESULT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-26</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formCompletedFieldName>cbc_complete</formCompletedFieldName><formDateField>cbc_lbdtc</formDateField><formImportedFieldName>cbc_nximport</formImportedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_redcap_units_value_field = etree.ElementTree(etree.fromstring(self.redcap_units_value_field))
        self.assertRaises(Exception, redi.copy_data_to_person_form_event_tree, self.data_redcap_field_units_name,
                          self.data_person_form_event_tree, self.data_form_event_tree)
        self.assertRaises(Exception, redi.copy_data_to_person_form_event_tree, self.data_redcap_units_value_field,
                          self.data_person_form_event_tree, self.data_form_event_tree)

    def test_Form_Completed_Field(self):
        self.form_event_tree = """<?xml version='1.0' encoding='US-ASCII'?>
        <redcapProject>
        <name>Project</name>
        <form>
            <name>inr</name>
            <formDateField>inr_lbdtc</formDateField>
            <formCompletedFieldName>inr_complete</formCompletedFieldName>
            <formImportedFieldName>inr_nximport</formImportedFieldName>
            <formCompletedFieldValue></formCompletedFieldValue>
            <formImportedFieldValue>Y</formImportedFieldValue>
            <event>
            <name>1_arm_1</name>
            </event>

           </form>

        </redcapProject>
        """
        self.data_form_event_tree = etree.ElementTree(etree.fromstring(self.form_event_tree))

        self.person_form_event_tree = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field>
            <field><name>inr_complete</name><value/></field>
            <field><name>inr_nximport</name><value/></field>
            </event>

        </form>
        </all_form_events></person></person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))
        self.Form_Completed_Field = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-26</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formDateField>inr_lbdtc</formDateField><formCompletedFieldName>inr_complete</formCompletedFieldName><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><formImportedFieldName>inr_nximport</formImportedFieldName><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_Form_Completed_Field = etree.ElementTree(etree.fromstring(self.Form_Completed_Field))
        self.assertRaises(Exception, redi.copy_data_to_person_form_event_tree, self.data_Form_Completed_Field,
                          self.data_person_form_event_tree, self.data_form_event_tree)

    def test_Form_Imported_Field(self):
        self.form_event_tree = """<?xml version='1.0' encoding='US-ASCII'?>
        <redcapProject>
        <name>Project</name>
        <form>
            <name>inr</name>
            <formDateField>inr_lbdtc</formDateField>
            <formCompletedFieldName>inr_complete</formCompletedFieldName>
            <formImportedFieldName>inr_nximport</formImportedFieldName>
            <formCompletedFieldValue>2</formCompletedFieldValue>
            <formImportedFieldValue></formImportedFieldValue>
            <event>
            <name>1_arm_1</name>
            </event>

           </form>

        </redcapProject>
        """
        self.data_form_event_tree = etree.ElementTree(etree.fromstring(self.form_event_tree))
        self.person_form_event_tree = """<person_form_event><person><study_id>456</study_id><all_form_events>
        <form>
            <name>inr</name>
            <event>
            <name>1_arm_1</name>
            <field><name>inr_lbdtc</name><value/></field>
            <field><name>inr_complete</name><value/></field>
            <field><name>inr_nximport</name><value/></field>
            </event>
        </form>
        </all_form_events></person></person_form_event>
        """
        self.data_person_form_event_tree = etree.ElementTree(etree.fromstring(self.person_form_event_tree))
        self.Form_Imported_Field = """<?xml version='1.0' encoding='US-ASCII'?>
            <study>
            <subject>
            <NAME>TestSubject_2</NAME>
            <RESULT>987</RESULT>
            <REFERENCE_UNIT>g/dL</REFERENCE_UNIT>
            <STUDY_ID>456</STUDY_ID>
        <timestamp>1906-12-26</timestamp><redcapFormName>inr</redcapFormName><eventName>1_arm_1</eventName><formCompletedFieldName>inr_complete</formCompletedFieldName><formDateField>inr_lbdtc</formDateField><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><formImportedFieldName>inr_nximport</formImportedFieldName><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits><redcapFieldNameStatus>hemo_lbstat</redcapFieldNameStatus></subject>

        </study>
            """
        self.data_Form_Imported_Field = etree.ElementTree(etree.fromstring(self.Form_Imported_Field))
        self.assertRaises(Exception, redi.copy_data_to_person_form_event_tree, self.data_Form_Imported_Field,
                          self.data_person_form_event_tree, self.data_form_event_tree)

    def test_form_imported_is_really_optional(self):
        """This test ensures formImportedFieldName is optional"""
        person_form_event_tree = """
            <person_form_event>
                <person>
                    <study_id>007</study_id>
                    <all_form_events>
                        <form>
                            <name>espionage</name>
                            <event>
                                <name>1_arm_1</name>
                                <field><name>interrogation</name><value /></field>
                                <field><name>interrogation_units</name><value /></field>
                                <field><name>espionage_complete</name><value/></field>
                            </event>
                        </form>
                    </all_form_events>
                </person>
            </person_form_event>
        """

        data_person_form_event_tree = etree.ElementTree(etree.fromstring(person_form_event_tree))

        study_data = """<?xml version="1.0" encoding="UTF-8"?>
            <study>
                <subject>
                    <NAME>Bond, James</NAME>
                    <RESULT>Passed</RESULT>
                    <REFERENCE_UNIT/>
                    <STUDY_ID>007</STUDY_ID>
                    <timestamp>1953-04-13</timestamp>
                    <redcapFormName>espionage</redcapFormName>
                    <eventName>1_arm_1</eventName>
                    <formDateField>date_taken</formDateField>
                    <formCompletedFieldName>espionage_complete</formCompletedFieldName>
                    <redcapFieldNameValue>interrogation</redcapFieldNameValue>
                    <redcapFieldNameUnits>interrogation_units</redcapFieldNameUnits>
                    <formImportedFieldName></formImportedFieldName>
                </subject>
            </study>"""

        data_one_subject = etree.ElementTree(etree.fromstring(study_data))

        form_event_tree = """<?xml version="1.0" encoding="UTF-8"?>
        <redcapProject>
            <name>Project</name>
            <form>
                <name>espionage</name>
                <formDateField>date_taken</formDateField>
                <formCompletedFieldName>espionage_complete</formCompletedFieldName>
                <formCompletedFieldValue>Completed</formCompletedFieldValue>
                <!-- THESE ARE SUPPOSED TO BE OPTIONAL:
                    <formImportedFieldValue>Y</formImportedFieldValue>
                    <formImportedFieldName>espionage_nximport</formImportedFieldName>
                -->
                <event>
                    <name>1_arm_1</name>
                </event>
            </form>
        </redcapProject>
        """
        data_form_event_tree = etree.ElementTree(etree.fromstring(form_event_tree))

        result = redi.copy_data_to_person_form_event_tree(data_one_subject,
                                                          data_person_form_event_tree,
                                                          data_form_event_tree)

        output = """
            <person_form_event>
                <person>
                    <study_id>007</study_id>
                    <all_form_events>
                        <form>
                            <name>espionage</name>
                            <event>
                                <name>1_arm_1</name>
                                <field><name>interrogation</name><value>Passed</value></field>
                                <field><name>interrogation_units</name><value /></field>
                                <field><name>espionage_complete</name><value>Completed</value></field>
                            </event>
                        </form>
                    </all_form_events>
                </person>
            </person_form_event>
        """

        expect = etree.tostring(etree.fromstring(output))
        self.assertEqual(expect, etree.tostring(result))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

'''
@author : Mohan
email : mohan88@ufl.edu

This file tests for the function parse_translation_table

'''
import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestParseTranslationTable(unittest.TestCase):

    def setUp(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.sampleData = """<rediFieldMap>
    <clinicalComponent>
        <clinicalComponentId>123456</clinicalComponentId>
        <clinicalComponentName>TestComponent</clinicalComponentName>
        <redcapFormName>Test</redcapFormName>
        <redcapFieldNameValue>TestField1</redcapFieldNameValue>
        <redcapFieldNameValueDescriptiveText>TestField</redcapFieldNameValueDescriptiveText>
        <redcapFieldNameUnits>TestField1Units</redcapFieldNameUnits>
        <redcapFieldNameUnitsDescriptiveText>TestFieldUnits_Description</redcapFieldNameUnitsDescriptiveText>
        <redcapStatusFieldName>TestFieldStatus</redcapStatusFieldName>
        <lbtest>dummytest</lbtest>
        <lbtestcd>dummytest</lbtestcd>
    </clinicalComponent>
    </rediFieldMap>
"""
        
    def test_parse_translation_table(self):
        file_name = "tempFile.xml"
        temp_xml = open(file_name, "wb")
        temp_xml.write(self.sampleData)
        temp_xml.close()
        result = redi.parse_raw_xml('tempFile.xml')
        result_str = etree.tostring(result, method='xml', pretty_print=True)
        self.assertEqual(self.sampleData, result_str)
        os.remove(file_name)
    
    def test_parse_raw_xml_without_file(self):
        self.assertRaises(redi.LogException,redi.parse_raw_xml,'')  
        
    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

"""
TestParseAll.py:

   Verifies the correct functionality 
   of all functions with prefix `parse_`
"""

import unittest
from lxml import etree
import logging
import os
from redi import redi

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestParseAll(unittest.TestCase):

    def setUp(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)

    ############################
    # == TEST_1 - config/formEvents.xml
    def test_parse_form_events(self):
        string_1_xml = """<?xml version="1.0" encoding="UTF-8"?>
<redcapProject>
	<name>Project</name>
	<form>
		<name>cbc</name>
		<formDateField>cbc_lbdtc</formDateField>
		<formCompletedFieldName>cbc_complete</formCompletedFieldName>
		<formImportedFieldName>cbc_nximport</formImportedFieldName>
        <formCompletedFieldValue>2</formCompletedFieldValue>
        <formImportedFieldValue>Y</formImportedFieldValue>
		<event>
    		<name>1_arm_1</name>
		</event>
		<event>
		    <name>2_arm_1</name>
		</event>
		<event>
		    <name>3_arm_1</name>
		</event>
	</form>
	<form>
		<name>chemistry</name>
		<formDateField>chem_lbdtc</formDateField>
		<formCompletedFieldName>chemistry_complete</formCompletedFieldName>
		<formImportedFieldName>chem_nximport</formImportedFieldName>
        <formCompletedFieldValue>2</formCompletedFieldValue>
        <formImportedFieldValue>Y</formImportedFieldValue>
		<event>
		    <name>1_arm_1</name>
		</event>
		<event>
		    <name>2_arm_1</name>
		</event>
    </form>
</redcapProject>
"""
        temp_file = 'tmp.xml'
        fh = open(temp_file, 'w')
        fh.write(string_1_xml)
        fh.close()

        etree_1 = etree.ElementTree(etree.fromstring(string_1_xml))
        result_tree = redi.parse_form_events(temp_file)
        result_xml  = etree.tostring(result_tree) 

        par = etree.XMLParser(remove_blank_text = True)
        clean_tree   = etree.XML(string_1_xml, parser = par)
        clean_result = etree.XML(result_xml, parser = par)

        self.assertEqual(etree.tostring(clean_tree), etree.tostring(clean_result))
        os.remove(temp_file)

    def tearDown(self):
        return()

    
if __name__ == "__main__":
    unittest.main()




#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import datetime
import unittest

from redi.utils import throttle


class TestThrottle(unittest.TestCase):

    def test_throttle(self):
        clock = MockClock()
        throttle.Throttle._now = clock
        throttle.Throttle._sleep = clock.add_seconds

        call = throttle.Throttle(lambda: None, max_calls=3,
                                 interval_in_seconds=5)

        call()
        call()
        call()
        self.assertEquals(3, len(call._timestamps))
        before = clock.now

        call()

        self.assertEqual(5, (clock.now - before).total_seconds())

    def test_throttle_front_loaded(self):
        clock = MockClock()
        throttle.Throttle._now = clock
        throttle.Throttle._sleep = clock.add_seconds

        call = throttle.Throttle(lambda: None, max_calls=3,
                                 interval_in_seconds=5)

        call()
        call()
        call()
        self.assertEquals(3, len(call._timestamps))
        clock.add_seconds(5)
        before = clock.now

        call()

        self.assertEquals(1, len(call._timestamps))
        self.assertEqual(0, (clock.now - before).total_seconds())

    def test_throttle_back_loaded(self):
        clock = MockClock()
        throttle.Throttle._now = clock
        throttle.Throttle._sleep = clock.add_seconds

        call = throttle.Throttle(lambda: None, max_calls=3,
                                 interval_in_seconds=5)

        start = clock.now
        call()
        clock.add_seconds(4)
        call()
        call()
        self.assertEquals(3, len(call._timestamps))

        call()
        self.assertEquals(3, len(call._timestamps))
        self.assertEqual(5, (clock.now - start).total_seconds())

        call()
        self.assertEquals(2, len(call._timestamps))
        self.assertEqual(9, (clock.now - start).total_seconds())


class MockClock(object):
    def __init__(self):
        self.now = datetime.datetime.now()

    def __call__(self):
        return self.now

    def add_seconds(self, seconds):
        self.now += datetime.timedelta(seconds=seconds)


if __name__ == '__main__':
    unittest.main()



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

"""
This module is used to connect to an sftp server
and retrieve the raw EMR file to be used as input for RED-I.
"""

import os
import csv
from xml.sax import saxutils
import logging
import pysftp
from csv2xml import openio, Writer
from paramiko.ssh_exception import SSHException, BadAuthenticationType
import sys

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

class EmrFileAccessDetails(object) :
    """
    Encapsulate the settings used to retrieve the EMR
    source file using an SFTP connection
    @see redi#_run()
    """
    def __init__(self,
            emr_download_file,
            emr_host,
            emr_username,
            emr_password,
            emr_port,
            emr_private_key,
            emr_private_key_pass
            ):

        self.download_file = emr_download_file
        self.host = emr_host
        self.username = emr_username
        self.password = emr_password
        self.port = int(emr_port)
        self.private_key = emr_private_key
        self.private_key_pass = emr_private_key_pass

#============================
# Module level functions
#============================

def download_file(destination, access_details):
    """
    Download a file from the sftp server
    :destination the name of the file which will be downloaded
    :access_details holds info for accessing the source file over sftp

    @see get_emr_data()
    """
    connection_info = dict(access_details.__dict__)
    # delete unnecessary element form the dictionary
    del connection_info['download_file']

    # check for errors during authentication with EMR server
    try:
        with pysftp.Connection(**connection_info) as sftp:
            logger.info("User %s connected to sftp server %s" % \
                (connection_info['username'], connection_info['host']))
            sftp.get(access_details.download_file, destination)
    except IOError as e:
        logger.error("Please verify that the private key file mentioned in "\
            "settings.ini exists.")
        logger.exception(e)
        sys.exit()
    except BadAuthenticationType as e:
        logger.error("Please verify that the EMR server connection details "\
            "under section emr_data in settings.ini are correct")
        logger.exception(e)
        sys.exit()
    except SSHException as e:
        logger.error("Please verify that the EMR server connection details "\
            "under section emr_data in settings.ini are correct")
        logger.exception(e)
        sys.exit()



def data_preprocessing(input_filename, output_filename):
    # replace &, >, < with &amp;, &>;, &<;
    with open(input_filename, 'r') as raw, open(output_filename, 'w') as processed:
        for line in raw:
            processed.write(saxutils.escape(line))


def generate_xml(input_filename, output_filename):

    # generate_xml now replicates the functionality from the
    # "main" code block of csv2xml.py. This allows us to use
    # it like another module in our project without having to call os.system().
    class Arguments:
        pass
    args = Arguments()

    # Set the properties which we used to pass as command line arguments
    args.iencoding = 'cp1252'
    args.oencoding = 'utf8'
    args.header = True,
    args.delimiter = ','
    args.declaration = True
    args.root_elem = 'study'
    args.record_elem = 'subject'
    args.ofile = output_filename
    args.ifile = input_filename

    # Now configure the defaults that would've been set if we were to execute
    # csv2xml.py from the command line.
    args.linebreak = u'\n'
    args.escapechar = None
    args.indent = u'    '
    args.quoting = csv.QUOTE_MINIMAL
    args.skipinitialspace = False
    args.field_elem = u'field'
    args.flat_fields = False
    args.doublequote = True
    args.quotechar = '"'
    args.newline_elem = None

    # WARNING! The rest of this function is copied verbatim from csv2xml.py.
    # There should be no differences between these blocks of code whatsoever.
    # TODO: Replace csv2xml.py entirely?
    csv.register_dialect('custom',
                         delimiter=args.delimiter,
                         doublequote=args.doublequote,
                         escapechar=args.escapechar,
                         quotechar=args.quotechar,
                         quoting=args.quoting,
                         skipinitialspace=args.skipinitialspace)
    with openio(args.ifile, mode='r', encoding=args.iencoding,
                newline='') as ifile:
        csvreader = csv.reader(ifile, dialect='custom')
        if args.header:
            args.header = next(csvreader)
        with openio(args.ofile, 'w', args.oencoding) as ofile:
            writer = Writer(ofile, args)
            writer.write_file(csvreader)

def cleanup(file_to_delete):
    os.remove(file_to_delete)

def get_emr_data(conf_dir, connection_details):
    """
    :conf_dir configuration directory name
    :connection_details EmrFileAccessDetails object
    """
    raw_txt_file = os.path.join(conf_dir, 'raw.txt')

    # download csv file
    download_file(raw_txt_file, connection_details)



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import tempfile

from lxml import etree

from redi.redi import PersonFormEventsRepository


class TestPersonFormEventsRepository(unittest.TestCase):

    def test_delete(self):
        with tempfile.NamedTemporaryFile(delete=False) as temp:
            service = PersonFormEventsRepository(temp.name)
            service.delete()

    def test_fetch(self):
        with tempfile.NamedTemporaryFile() as temp:
            temp.write('<test>42</test>')
            temp.seek(0)

            service = PersonFormEventsRepository(temp.name)
            xml = service.fetch()
            self.assertEquals(42, int(xml.xpath('//test')[0].text))

    def test_store(self):
        class MockLogger(object):
            @staticmethod
            def debug(*args):
                MockLogger.message = repr(args)

        with tempfile.NamedTemporaryFile() as temp:
            xml = etree.fromstring('<test>42</test>').getroottree()
            service = PersonFormEventsRepository(temp.name, MockLogger())

            service.store(xml)

            self.assertTrue('<test>42</test>' in open(temp.name).read())
            self.assertIsNotNone(MockLogger.message)


# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

"""
Utility module for throttling calls to a function
"""

import collections
import datetime
import logging
import time

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class Throttle(object):
    """
    Limits the number of calls to a function to a given rate.

    The rate limit is equal to the max_calls over the interval_in_seconds.

    :param function: function to call after throttling
    :param max_calls: maximum number of calls allowed
    :param interval_in_seconds: size of the sliding window
    """
    def __init__(self, function, max_calls, interval_in_seconds=60):
        assert max_calls > 0
        assert interval_in_seconds > 0

        self._actual = function
        self._max_requests = max_calls
        self._interval = datetime.timedelta(seconds=interval_in_seconds)
        self._timestamps = collections.deque(maxlen=self._max_requests)

    def __call__(self, *args, **kwargs):
        """ Conditionally delays before calling the function """
        self._wait()
        return self._actual(*args, **kwargs)

    def _limit_reached(self):
        """ Returns True if the maximum number of calls has been reached """
        return len(self._timestamps) == self._max_requests

    @staticmethod
    def _now():
        # Used during unit testing
        return datetime.datetime.now()

    def _remove_old_entries(self):
        """ Removes old timestamp entries """
        # @TODO: investigate why the deque is not
        # cleared after we reach the limit

        #logger.debug("Clear the deque")
        #self._timestamps.clear()
        #return

        while (len(self._timestamps) > 0 and
               self._now() - self._timestamps[0] >= self._interval):
            self._timestamps.popleft()

    @staticmethod
    def _sleep(seconds):
        # Used during unit testing
        return time.sleep(seconds)

    def _wait(self):
        """ Sleeps for the remaining interval if the limit has been reached """
        if self._limit_reached():
            logger.warn('Throttling limit {} reached.' \
                    .format(self._max_requests))
            lapsed = self._now() - self._timestamps[0]

            if lapsed < self._interval:
                sleep_time = (self._interval - lapsed).total_seconds()
                logger.debug("Sleeping for {} seconds to prevent too many calls"
                             .format(sleep_time))
                self._sleep(sleep_time)

            self._remove_old_entries()

        self._timestamps.append(self._now())



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
from lxml import etree
from redi import redi


class TestUpdateRedcapForm(unittest.TestCase):

    def setUp(self):
        self.rawXml = """<?xml version='1.0' encoding='US-ASCII'?>
<study>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>04/17/19</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMATOCRIT</Component_Name>
        <loinc_code>1534436</loinc_code>
        <Reference_Unit>%</Reference_Unit>
        <Result_Value>34.5</Result_Value>
    <timestamp/><redcapFormName/><eventName/><formDateField/><formCompletedFieldName/></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>04/17/19</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMOGLOBIN</Component_Name>
        <loinc_code>1534435</loinc_code>
        <Reference_Unit>g/dL</Reference_Unit>
        <Result_Value>11.3</Result_Value>
    <timestamp/><redcapFormName/><eventName/><formDateField/><formCompletedFieldName/></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>09/12/22</Collection_Date>
        <Collection_Time>12:38</Collection_Time>
        <Component_Name>BILIRUBIN DIRECT</Component_Name>
        <loinc_code>1558221</loinc_code>
        <Reference_Unit>mg/dL</Reference_Unit>
        <Result_Value>0.8</Result_Value>
    <timestamp/><redcapFormName/><eventName/><formDateField/><formCompletedFieldName/></subject>
</study>
"""
        self.data = etree.ElementTree(etree.fromstring(self.rawXml))

        self.translationTable = """<rediFieldMap>
    <clinicalComponent>
        <loinc_code>1534435</loinc_code>
        <clinicalComponentName>HEMOGLOBIN</clinicalComponentName>
        <redcapFormName>cbc</redcapFormName>
        <redcapFieldNameValue>hemo_lborres</redcapFieldNameValue>
        <redcapFieldNameValueDescriptiveText>Hemoglobin</redcapFieldNameValueDescriptiveText>
        <redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits>
        <redcapFieldNameUnitsDescriptiveText>Hemoglobin units</redcapFieldNameUnitsDescriptiveText>
        <lbtest>hemo_lbtest</lbtest>
        <lbtestcd>hemo_lbtestcd</lbtestcd>
    </clinicalComponent>
    <clinicalComponent>
        <loinc_code>1558221</loinc_code>
        <clinicalComponentName>BILIRUBIN DIRECT</clinicalComponentName>
        <redcapFormName>chemistry</redcapFormName>
        <redcapFieldNameValue>dbil_lborres</redcapFieldNameValue>
        <redcapFieldNameValueDescriptiveText>Direct Bilirubin</redcapFieldNameValueDescriptiveText>
        <redcapFieldNameUnits>dbil_lborresu</redcapFieldNameUnits>
        <redcapFieldNameUnitsDescriptiveText>Direct Bilirubin units</redcapFieldNameUnitsDescriptiveText>
        <lbtest>Total Bilirubin</lbtest>
        <lbtestcd>BILI</lbtestcd>
    </clinicalComponent>
</rediFieldMap>
"""

        self.translationTableTree = etree.ElementTree(etree.fromstring(self.translationTable))

        self.output = """<?xml version='1.0' encoding='US-ASCII'?>
<study>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>04/17/19</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMATOCRIT</Component_Name>
        <loinc_code>1534436</loinc_code>
        <Reference_Unit>%</Reference_Unit>
        <Result_Value>34.5</Result_Value>
    <timestamp/><redcapFormName>undefined</redcapFormName><eventName/><formDateField/><formCompletedFieldName/></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>04/17/19</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMOGLOBIN</Component_Name>
        <loinc_code>1534435</loinc_code>
        <Reference_Unit>g/dL</Reference_Unit>
        <Result_Value>11.3</Result_Value>
    <timestamp/><redcapFormName>cbc</redcapFormName><eventName/><formDateField/><formCompletedFieldName/></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>09/12/22</Collection_Date>
        <Collection_Time>12:38</Collection_Time>
        <Component_Name>BILIRUBIN DIRECT</Component_Name>
        <loinc_code>1558221</loinc_code>
        <Reference_Unit>mg/dL</Reference_Unit>
        <Result_Value>0.8</Result_Value>
    <timestamp/><redcapFormName>chemistry</redcapFormName><eventName/><formDateField/><formCompletedFieldName/></subject>
</study>
"""
        self.expect = etree.tostring(etree.fromstring(self.output))
        return()


    def test_updateRedcapForm(self):
        redi.update_redcap_form(self.data, self.translationTableTree, 'undefined')
        result = etree.tostring(self.data)
        self.assertEqual(self.expect, result)


    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'


class TestUpdateRedcapFieldNameValueAndUnits(unittest.TestCase):

    def setUp(self):
        self.raw_xml = """<study>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>10/01/17</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMATOCRIT</Component_Name>
        <loinc_code>1534436</loinc_code>
        <Reference_Unit>%</Reference_Unit>
        <Result_Value>34.5</Result_Value>
<timestamp/><redcapFormName/><eventName/><formDateField/><formCompletedFieldName/><redcapFieldNameValue/><redcapFieldNameUnits/></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>10/01/17</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMOGLOBIN</Component_Name>
        <loinc_code>1534435</loinc_code>
        <Reference_Unit>g/dL</Reference_Unit>
        <Result_Value>11.3</Result_Value>
<timestamp/><redcapFormName/><eventName/><formDateField/><formCompletedFieldName/><redcapFieldNameValue/><redcapFieldNameUnits/></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>05/05/22</Collection_Date>
        <Collection_Time>12:38</Collection_Time>
        <Component_Name>BILIRUBIN DIRECT</Component_Name>
        <loinc_code>1558221</loinc_code>
        <Reference_Unit>mg/dL</Reference_Unit>
        <Result_Value>0.8</Result_Value>
<timestamp/><redcapFormName/><eventName/><formDateField/><formCompletedFieldName/><redcapFieldNameValue/><redcapFieldNameUnits/></subject>
</study>
"""
        self.data_tree = etree.ElementTree(etree.fromstring(self.raw_xml))

        self.lookup_table = """<rediFieldMap>
    <clinicalComponent>
        <loinc_code>1534435</loinc_code>
        <clinicalComponentName>HEMOGLOBIN</clinicalComponentName>
        <redcapFormName>cbc</redcapFormName>
        <redcapFieldNameValue>hemo_lborres</redcapFieldNameValue>
        <redcapFieldNameValueDescriptiveText>Hemoglobin</redcapFieldNameValueDescriptiveText>
        <redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits>
        <redcapFieldNameUnitsDescriptiveText>Hemoglobin units</redcapFieldNameUnitsDescriptiveText>
        <lbtest>hemo_lbtest</lbtest>
        <lbtestcd>hemo_lbtestcd</lbtestcd>
    </clinicalComponent>
    <clinicalComponent>
        <loinc_code>1558221</loinc_code>
        <clinicalComponentName>BILIRUBIN DIRECT</clinicalComponentName>
        <redcapFormName>chemistry</redcapFormName>
        <redcapFieldNameValue>dbil_lborres</redcapFieldNameValue>
        <redcapFieldNameValueDescriptiveText>Direct Bilirubin</redcapFieldNameValueDescriptiveText>
        <redcapFieldNameUnits>dbil_lborresu</redcapFieldNameUnits>
        <redcapFieldNameUnitsDescriptiveText>Direct Bilirubin units</redcapFieldNameUnitsDescriptiveText>
        <lbtest>Total Bilirubin</lbtest>
        <lbtestcd>BILI</lbtestcd>
    </clinicalComponent>
</rediFieldMap>
"""


        self.lookup_table_tree = etree.ElementTree(etree.fromstring(self.lookup_table))

        self.output = """<study>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>10/01/17</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMATOCRIT</Component_Name>
        <loinc_code>1534436</loinc_code>
        <Reference_Unit>%</Reference_Unit>
        <Result_Value>34.5</Result_Value>
<timestamp/><redcapFormName/><eventName/><formDateField/><formCompletedFieldName/><redcapFieldNameValue>undefined</redcapFieldNameValue><redcapFieldNameUnits>redcapFieldNameUnitsUndefined</redcapFieldNameUnits></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>10/01/17</Collection_Date>
        <Collection_Time>13:50</Collection_Time>
        <Component_Name>HEMOGLOBIN</Component_Name>
        <loinc_code>1534435</loinc_code>
        <Reference_Unit>g/dL</Reference_Unit>
        <Result_Value>11.3</Result_Value>
<timestamp/><redcapFormName/><eventName/><formDateField/><formCompletedFieldName/><redcapFieldNameValue>hemo_lborres</redcapFieldNameValue><redcapFieldNameUnits>hemo_lborresu</redcapFieldNameUnits></subject>
    <subject>
        <Study_Id>22</Study_Id>
        <Collection_Date>05/05/22</Collection_Date>
        <Collection_Time>12:38</Collection_Time>
        <Component_Name>BILIRUBIN DIRECT</Component_Name>
        <loinc_code>1558221</loinc_code>
        <Reference_Unit>mg/dL</Reference_Unit>
        <Result_Value>0.8</Result_Value>
<timestamp/><redcapFormName/><eventName/><formDateField/><formCompletedFieldName/><redcapFieldNameValue>dbil_lborres</redcapFieldNameValue><redcapFieldNameUnits>dbil_lborresu</redcapFieldNameUnits></subject>
</study>
"""
        self.expect = etree.tostring(etree.fromstring(self.output))
        return()


    def test_update_redcap_field_name_value_and_units(self):
        redi.update_redcap_field_name_value_and_units(self.data_tree, self.lookup_table_tree, 'undefined')
        self.result = etree.tostring(self.data_tree)
        self.assertEqual(self.expect, self.result)

    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

"""
TestCreateImportDataJson.py:

   Verifies the correct functionality 
   of the `test_create_import_data_json` function
"""

import unittest
from lxml import etree
import logging
import os
from redi import redi
from redi import upload

DEFAULT_DATA_DIRECTORY = os.getcwd()


class TestCreateImportDataJson(unittest.TestCase):

    def setUp(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.CONST_STUDY_ID = 73
        return()

    ############################
    # == TEST_1
    def test_empty_event(self):
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        logging.info("Running " + __name__ 
            + "#test_empty_event() using study_id: " + `self.CONST_STUDY_ID`)
        # Case 1 input string
        string_1_empty_event = """
<event>
</event>
"""
        out_dict_1 = {'study_id':self.CONST_STUDY_ID}
        etree_1 = etree.ElementTree(etree.fromstring(string_1_empty_event))
        self.assertRaises(Exception, upload.create_import_data_json, out_dict_1, etree_1)
        


    ############################
    # == TEST_2
    def test_empty_event_field_value(self):
        logging.info("Running " + __name__ 
            + "#test_empty_value() for study_id: " + `self.CONST_STUDY_ID`)
        # Case 2 input string
        string_2_empty_values = """
<event>
    <name>1_arm_1</name>
    <field>
        <name>chem_lbdtc</name>
        <value/>
    </field>
    <field>
        <name>chem_complete</name>
        <value/>
    </field>
    <field>
        <name>chem_nximport</name>
        <value/>
    </field>
    <field>
        <name>tbil_lborres</name>
        <value/>
    </field>
    <field>
        <name>tbil_lborresu</name>
        <value/>
    </field>
</event>"""

        string_2_out = """
record,redcap_event_name,field_name,value
73,"1_arm_1",chem_lbdtc,""
73,"1_arm_1",chem_complete,""
73,"1_arm_1",chem_nximport,""
73,"1_arm_1",tbil_lborres,""
73,"1_arm_1",tbil_lborresu,""
"""

        etree_2 = etree.ElementTree(etree.fromstring(string_2_empty_values))
        out_dict_2 = {'study_id':self.CONST_STUDY_ID}
        expected_result_dict_2 = {'contains_data': False, 'json_data': {'chem_complete': '', 'redcap_event_name': '1_arm_1', 'tbil_lborres': '', 'study_id': 73, 'chem_nximport': '', 'tbil_lborresu': '', 'chem_lbdtc': ''}}
        actual_result = upload.create_import_data_json(out_dict_2, etree_2)
        self.assertEqual(expected_result_dict_2,actual_result)


    ############################
    # == Test_3
    def test_mixed_event_field_value(self):
        logging.info("Running " + __name__ 
            + "#test_mixed_event_field_value() for study_id: " + `self.CONST_STUDY_ID`)


        string_3_mixed = """
<event>
    <name>1_arm_1</name>
    <field>
        <name>chem_lbdtc</name>
        <value>1902-12-17</value>
    </field>
    <field>
        <name>chem_complete</name>
        <value>2</value>
    </field>
    <field>
        <name>chem_nximport</name>
        <value>Y</value>
    </field>
    <field>
        <name>tbil_lborres</name>
        <value>1.7</value>
    </field>
    <field>
        <name>tbil_lborresu</name>
        <value/>
    </field>
</event>"""

        string_3_out = """
record,redcap_event_name,field_name,value
73,"1_arm_1",chem_lbdtc,"1902-12-17"
73,"1_arm_1",chem_complete,"2"
73,"1_arm_1",chem_nximport,"Y"
73,"1_arm_1",tbil_lborres,"1.7"
73,"1_arm_1",tbil_lborresu,""
"""
        etree_3 = etree.ElementTree(etree.fromstring(string_3_mixed))
        out_dict_3 = {'study_id':self.CONST_STUDY_ID}
        actual_result = upload.create_import_data_json(out_dict_3, etree_3)
        expected_result = {'contains_data': True, 'json_data': {'chem_complete': '2', 'redcap_event_name': '1_arm_1', 'tbil_lborres': '1.7', 'study_id': 73, 'chem_nximport': 'Y', 'tbil_lborresu': '', 'chem_lbdtc': '1902-12-17'}}
        self.assertEqual(actual_result, expected_result)


    ############################
    # == TEST_4
    def test_empty_event_field_name(self):
        logging.info("Running " + __name__ 
            + "#test_empty_event_field_name() for study_id: " + `self.CONST_STUDY_ID`)

        # Case 4 input string
        string_4_blank_name = """
<event>
    <name>1_arm_1</name>
    <field>
        <name>chem_lbdtc</name>   <!-- valid text -->
        <value>1902-12-17</value>
    </field>
    <field>
        <name></name>             <!-- missing text -->
        <value>2</value>
    </field>
    <field>
        <name>chem_nximport</name>
        <value>Y</value>
    </field>
    <field>
        <name>tbil_lborres</name>
        <value>1.7</value>
    </field>
    <field>
        <name>tbil_lborresu</name>
        <value/>
    </field>
</event> """
        etree_4 = etree.ElementTree(etree.fromstring(string_4_blank_name))
        out_dict_4 = {'study_id':self.CONST_STUDY_ID}
        self.assertRaises(Exception, upload.create_import_data_json, out_dict_4, etree_4)

    # Verify if code checks for blank `event/name`
    def test_empty_event_name(self):
        logging.info("Running " + __name__
            + "#test_empty_event_name() for study_id: " + `self.CONST_STUDY_ID`)

        string_4a_blank_name = """
<event>
    <!-- <name></name> -->
    <name></name>
    <field>
        <name>tbil_lborresu</name>
    </field>
</event> """
        etree_4a = etree.ElementTree(etree.fromstring(string_4a_blank_name))
        self.assertRaises(Exception, upload.create_import_data_json, self.CONST_STUDY_ID, etree_4a)

    ############################
    # == TEST_5
    def test_empty_study_id(self) :
        logging.info("Running " + __name__ 
            + "#test_empty_study_id() for study_id: ''")
        string_1_empty_event = """
<event></event>
"""
        string_5_out = "error_study_id_empty"
        etree_1 = etree.ElementTree(etree.fromstring(string_1_empty_event))
        self.assertRaises(Exception, upload.create_import_data_json,None, etree_1)

    def test_multiple_event(self):
        # motivated by bug 5996
        form = etree.fromstring(
            "<form>"
            "  <event><name>42_arm_42</name>"
            "    <field><name>foo</name><value>bar</value></field>"
            "  </event>"
            "  <event><name>no_arm</name>"
            "    <field><name>foo</name><value>bar</value></field>"
            "  </event>"
            "</form>")

        second_event = form.xpath('//event')[1]
        out_dict_3 = {'study_id':self.CONST_STUDY_ID}

        output = upload.create_import_data_json(out_dict_3, second_event)
        self.assertTrue(output['contains_data'])
        self.assertFalse('42_arm_42' in output['json_data']['redcap_event_name'])
        self.assertTrue('no_arm' in output['json_data']['redcap_event_name'])

    def tearDown(self):
        return()

    
if __name__ == "__main__":
    unittest.main()




#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import unittest
import os
import tempfile
from redi import redi
import shutil


class TestGetDBPath(unittest.TestCase):

    def setUp(self):
        pass

    def test_get_db_path(self):
        db_path = tempfile.mkdtemp()
        batch_info_database = "database.db"
        redi.get_db_path(batch_info_database, db_path)
        full_db_path = os.path.join(batch_info_database, db_path)
        assert os.path.exists(full_db_path) == 1
        shutil.rmtree(full_db_path)

    def test_get_db_path_not_exists(self):
        # verify that db path is created if it doesn't exist
        db_path =  "nonExistentPath"
        batch_info_database = "database.db"
        created_db_path = redi.get_db_path(batch_info_database, db_path)
        assert created_db_path ==  os.path.join(db_path, batch_info_database)
        shutil.rmtree(db_path)

if __name__ == '__main__':
    unittest.main()



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import logging
from HTMLParser import HTMLParser
import urllib
import urllib2
"""
Usage:
    Below is the sample code to use this utility.

    from url_generator import url_generator

    url = "http://localhost:8998/redcap/plugins/show_url.php"
    values = {"project_name":"Classic Database",
              "study_id":"999-001",
              "page_name":"demographics",
              "event_name":"Event 1"}
    parser = InitPlugin(url, values)


"""
# Configure module's logger
logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

class InitPlugin():
    """Call the Form URL Plugin for REDCap with POST parameters
    and get the response.
    Parse the response and extract the URL form it.

    """
    def __init__(self, url, values):
        self.url = url
        self.values = values
        log_str = "Initializing Plugin parser with URL: "+self.url
        logger.info(log_str)
        self.get_response(self.url, self.values)

    def get_response(self, url, values):
        post_data = urllib.urlencode(values)
        prepared_request = urllib2.Request(url, post_data)
        response = urllib2.urlopen(prepared_request)
        html_string = response.read()
        parser = PluginParser()
        parser.feed(html_string)
        return parser.output

class PluginParser(HTMLParser):
    """Parser module for the HTML response received

    """
    def feed(self, data):
        self.output = []
        HTMLParser.feed(self, data)

    def handle_starttag(self, tag, attrs):
        # search for tag 'a'
        if tag == "a":
            # iterate through properties of anchor tag
            for name, value in attrs:
                if name == "href":
                    self.output.append(value)



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

'''
@author : Radha
email : rkandula@ufl.edu

This file tests for the function updateTimeStamp.
we call the function and check if the function updates the timestamps of all the subjects
if all the subject's timestamps are updated the status value stays 1
else the status value is flipped to 0 and loop is exited and Hence the test fails in
assertion

'''
import unittest
import os
from lxml import etree
from redi import redi

file_dir = os.path.dirname(os.path.realpath(__file__))
goal_dir = os.path.join(file_dir, "../")
proj_root = os.path.abspath(goal_dir)+'/'

DEFAULT_DATA_DIRECTORY = os.getcwd()

class TestUpdateTimestamp(unittest.TestCase):

    def setUp(self):
        # initialize the data with element tree
        redi.configure_logging(DEFAULT_DATA_DIRECTORY)
        self.test_xml = """<?xml version="1.0" encoding="utf8"?>
<study>
    <subject>
        <NAME>TSH REFLEX</NAME>
        <loinc_code>1552152</loinc_code>
        <RESULT>0.74</RESULT>
        <REFERENCE_LOW>0.27</REFERENCE_LOW>
        <REFERENCE_HIGH>4.20</REFERENCE_HIGH>
        <REFERENCE_UNIT>mIU/L</REFERENCE_UNIT>
        <DATE_TIME_STAMP>1903-11-27 15:13:00</DATE_TIME_STAMP>
        <STUDY_ID>59</STUDY_ID>
    </subject>
    <subject>
        <NAME>HEP C RNA, QUANT REAL-TIME</NAME>
        <loinc_code>740</loinc_code>
        <RESULT>5.8</RESULT>
        <REFERENCE_LOW></REFERENCE_LOW>
        <REFERENCE_HIGH></REFERENCE_HIGH>
        <REFERENCE_UNIT>log IU</REFERENCE_UNIT>
        <DATE_TIME_STAMP>1903-11-27 15:13:00</DATE_TIME_STAMP>
        <STUDY_ID>59</STUDY_ID>
    </subject>
    <subject>
        <NAME>HCV QUANTITATIVE INTERPRETATION</NAME>
        <loinc_code>1534483</loinc_code>
        <RESULT>Detected</RESULT>
        <REFERENCE_LOW></REFERENCE_LOW>
        <REFERENCE_HIGH></REFERENCE_HIGH>
        <REFERENCE_UNIT></REFERENCE_UNIT>
        <DATE_TIME_STAMP>1903-11-27 15:13:00</DATE_TIME_STAMP>
        <STUDY_ID>59</STUDY_ID>
    </subject>
    </study>"""
        self.data = etree.ElementTree(etree.fromstring(self.test_xml))
        self.input_date_format = "%Y-%m-%d %H:%M:%S"
        self.output_date_format = "%Y-%m-%d"

    def test_updateTimeStamp(self):
	    # add blank elements to each subject in data tree
        redi.add_elements_to_tree(self.data)

        # function to be tested
        redi.update_time_stamp(self.data, self.input_date_format, self.output_date_format)

	    # output raw file to check it
        #redi.write_element_tree_to_file(self.data, 'rawData.xml')

        #initialize a dictionary for the timestamps
        # key,value = timestamp, filled or not?(0/1)
        isStampFilled = {}
        for subject in self.data.iter('subject'):
            ts = subject.find('timestamp').text
            if not ts:
                isStampFilled[ts] = 1
        status=1
        for key,value in isStampFilled.items():
            if value != status:
                status = 0
                break
        self.assertEqual(status,1)
    
    def tearDown(self):
        return()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

'''
This file creates a test suite for all the test classes.

    IMPORTANT: the imports should be updated in order to add the test to
    the test suite.
    Notes:
        - No error will be thrown if the `import` statement is missing
        - No test will be run if the `import` statement is missing even
            if we add the line `addTest(<test_class>)`
        - To run the script the user must instal a few libraries
            On a mac: pip install lxml request
'''
import unittest
from TestReadConfig import TestReadConfig
from TestWriteToFile import TestWriteToFile
from TestUpdateRedcapForm import TestUpdateRedcapForm
from TestUpdateTimestamp import TestUpdateTimestamp
from TestUpdateFormDateField import TestUpdateFormDateField
from TestUpdateFormCompletedFieldName import TestUpdateFormCompletedFieldName
from TestSortElementTree import TestSortElementTree
from TestCompressDataUsingStudyFormDate import TestCompressDataUsingStudyFormDate

from TestUpdateDataFromLookup import TestUpdateDataFromLookup
from TestAddElementsToTree import TestAddElementsToTree
from TestUpdateRedcapFieldNameValueAndUnits import TestUpdateRedcapFieldNameValueAndUnits
from TestUpdateEventName import TestUpdateEventName
from TestResearchIdToRedcapId import TestResearchIdToRedcapId
from TestUpdateFormImported import TestUpdateFormImported
from TestCreateSummaryReport import TestCreateSummaryReport
from TestUpdateStatusField import TestUpdateStatusField
from TestCreateEmptyEventsForOneSubject import TestCreateEmptyEventsForOneSubject
from TestCreateEmptyEventTreeForStudy import TestCreateEmptyEventTreeForStudy
from TestCreateImportDataJson import TestCreateImportDataJson
from TestGenerateOutput import TestGenerateOutput
from TestParseAll import TestParseAll
from TestHandleREDCapResponse import TestHandleErrorsInREDCapResponse
from TestParseRawXml import TestParseRawXml
from TestValidateXmlFleAndExtractData import TestValidateXmlFleAndExtractData
from TestConvertComponentIdToLoincCode import TestConvertComponentIdToLoincCode
from TestCopyDataToPersonFormEventTree import TestCopyDataToPersonFormEventTree
from TestGetEMRData import TestGetEMRData
from TestResume import TestResume
from TestThrottle import TestThrottle
from TestPersonFormEventsRepository import TestPersonFormEventsRepository
from TestVerifyAndCorrectCollectionDate import TestVerifyAndCorrectCollectionDate
from TestRediEmail import TestRediEmail
from TestSentEventIndex import TestSentEventIndex
from TestSendDatatoRedcap import TestSendDatatoRedcap
from TestGetDBPath import TestGetDBPath

class redi_suite(unittest.TestSuite):

    def suite(self):
        # create a test suite
        redi_test_suite = unittest.TestSuite()

        # add the test to the suite in the order to be tested
        redi_test_suite.addTest(TestReadConfig)
        redi_test_suite.addTest(TestWriteToFile)
        redi_test_suite.addTest(TestUpdateRedcapForm)
        redi_test_suite.addTest(TestUpdateTimestamp)
        redi_test_suite.addTest(TestUpdateFormDateField)
        redi_test_suite.addTest(TestSortElementTree)
        redi_test_suite.addTest(TestCompressDataUsingStudyFormDate)
        redi_test_suite.addTest(TestUpdateDataFromLookup)
        redi_test_suite.addTest(TestAddElementsToTree)
        redi_test_suite.addTest(TestUpdateRedcapFieldNameValueAndUnits)
        redi_test_suite.addTest(TestUpdateEventName)
        redi_test_suite.addTest(TestResearchIdToRedcapId)
        redi_test_suite.addTest(TestUpdateFormImported)
        redi_test_suite.addTest(TestCreateSummaryReport)
        redi_test_suite.addTest(TestUpdateFormCompletedFieldName)
        redi_test_suite.addTest(TestUpdateStatusField)
        redi_test_suite.addTest(TestCreateEmptyEventsForOneSubject)
        redi_test_suite.addTest(TestCreateEmptyEventTreeForStudy)
        redi_test_suite.addTest(TestVerifyAndCorrectCollectionDate)
        redi_test_suite.addTest(TestParseAll)
        redi_test_suite.addTest(TestSentEventIndex)

        # The redesign functions May 2014
        redi_test_suite.addTest(TestCreateImportDataJson)
        redi_test_suite.addTest(TestGenerateOutput)
        redi_test_suite.addTest(TestHandleErrorsInREDCapResponse)
        redi_test_suite.addTest(TestParseRawXml)
        redi_test_suite.addTest(TestValidateXmlFleAndExtractData)
        redi_test_suite.addTest(TestConvertComponentIdToLoincCode)
        redi_test_suite.addTest(TestCopyDataToPersonFormEventTree)
        redi_test_suite.addTest(TestGetEMRData)
        redi_test_suite.addTest(TestResume)
        redi_test_suite.addTest(TestThrottle)
        redi_test_suite.addTest(TestPersonFormEventsRepository)
        redi_test_suite.addTest(TestRediEmail)
        redi_test_suite.addTest(TestDaysSinceToday)
        redi_test_suite.addTest(TestSendDatatoRedcap)

        redi_test_suite.addTest(TestGetDBPath)

        # return the suite
        return unittest.TestSuite([redi_test_suite])

def main():
    unittest.main(buffer=True)


if __name__ == '__main__':
    main()



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Nicholas Rejack <nrejack@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

import abc
import time

import pkg_resources
from datetime import datetime, timedelta
from lxml import etree

from utils import redi_email

REDI_PACKAGE_NAME = 'redi'


class ReportCourier(object):
    @abc.abstractmethod
    def deliver(self, report):
        raise NotImplementedError()


class ReportFileWriter(ReportCourier):
    def __init__(self, output_file, logger):
        self._output_file = output_file
        self._logger = logger

    def deliver(self, report):
        """
        Deliver the summary report by writing it to a file
        or logging it to the console if writing the file fails

        :html_report_path the path where the report will be stored
        :html the actual report content
        """
        logger = self._logger
        html_report_path = self._output_file
        html = report

        problem_found = False
        try:
            report_file = open(html_report_path, 'w')
        except (IOError, OSError) as e:
            logger.exception('Could not open file: %s' % html_report_path)
            problem_found = True
        else:
            try:
                report_file.write(html)
                logger.info("==> You can review the summary report by opening: {}"\
                    " in your browser".format(html_report_path))
            except IOError:
                logger.exception('Could not write file: %s' % html_report_path)
                problem_found = True
            finally:
                report_file.close()
        if problem_found:
            logger.info("== Summary report ==" + html)


class ReportEmailSender(ReportCourier):
    def __init__(self, settings, logger):
        self._settings = settings
        self._logger = logger

    def deliver(self, report):
        """
        Deliver summary report as an email

        :email_settings dictinary with email parameters
        :html the actual report content
        """
        logger = self._logger
        email_settings = self._settings
        html = report
        # TODO: Replace this with a "backup_courier" constructor injection
        deliver_report_as_file = self.__backup_courier_wrapper()

        try:
            redi_email.send_email_data_import_completed(email_settings, html)
            logger.info("Summary report was emailed: parameter 'send_email = Y'")
        except Exception as e:
            logger.error("Unable to deliver the summary report due error: %s" % e)
            deliver_report_as_file("report.html", html)

    def __backup_courier_wrapper(self):
        # Needed for backwards-compatibility with the old
        # deliver_report_by_file() call from deliver_report_by_email()
        logger = self._logger

        def wrapper(filename, report):
            return ReportFileWriter(filename, logger).deliver(report)

        return wrapper


class ReportCreator(object):
    def __init__(self, report_file_path, project_name, redcap_uri,
                 sort_by_lab_id, writer):
        self._report_parameters = {
            'report_file_path': report_file_path,
            'project': project_name,
            'redcap_uri': redcap_uri,
            'is_sort_by_lab_id': sort_by_lab_id
        }
        self._writer = writer

    def create_report(self, report_data, alert_summary, collection_date_summary_dict, duration_dict):
        report_parameters = self._report_parameters
        write_element_tree_to_file = self._writer

        root = etree.Element("report")
        root.append(etree.Element("header"))
        root.append(etree.Element("summary"))
        root.append(etree.Element("alerts"))
        root.append(etree.Element("subjectsDetails"))
        root.append(etree.Element("errors"))
        root.append(etree.Element("summaryOfSpecimenTakenTimes"))
        updateReportHeader(root, report_parameters)
        updateReportSummary(root, report_data)
        updateSubjectDetails(root, report_data['subject_details'])
        updateReportAlerts(root, alert_summary)
        updateReportErrors(root, report_data['errors'])
        updateSummaryOfSpecimenTakenTimes(root, collection_date_summary_dict)

        # TODO: remove dependency on the order of the xml elements in the report
        sort_by_value = 'lab_id' if report_parameters['is_sort_by_lab_id'] else 'redcap_id'
        root.append(gen_ele("sort_details_by", sort_by_value))

        start = duration_dict['all']['start']
        end = duration_dict['all']['end']
        diff = self.get_time_diff(end, start)
        root.append(gen_ele('time_all_start', start[-8:]))
        root.append(gen_ele('time_all_end', end[-8:]))
        root.append(gen_ele('time_all_diff', self.format_seconds_as_string(diff)))

        tree = etree.ElementTree(root)
        write_element_tree_to_file(tree,report_parameters.get('report_file_path'))

        report_xsl = pkg_resources.resource_filename(REDI_PACKAGE_NAME,
                                                     'utils/report.xsl')
        xslt = etree.parse(report_xsl)
        transform = etree.XSLT(xslt)
        html_report = transform(tree)
        html_str = etree.tostring(html_report, method='html', pretty_print=True)

        return html_str

    def get_time_diff(self, end, start):
        """
        Get time difference in seconds from the two dates
        Parameters
        ----------
        end : string
            The end timestamp
        start : string
            The start timestamp
        """
        # sqlite: select strftime('%s', rbEndTime) - strftime('%s', rbStartTime) from RediBatch;
        fmt = '%Y-%m-%d %H:%M:%S'
        dt_end = datetime.strptime(end, fmt)
        dt_start = datetime.strptime(start, fmt)
        diff = (dt_end - dt_start).total_seconds()
        return diff


    def format_seconds_as_string(self,seconds):
        """
        Convert seconds to a friendly strings
            3662  ==> '01:01:02'
            89662 ==> '1 day, 0:54:22'
        Parameters
        ----------
        seconds : integer
            The number of seconds to be converted
        """
        return str(timedelta(seconds=seconds))


def updateReportHeader(root, report_parameters):
    """ Update the passed `root` element tree with date, project name and url"""
    header = root[0]
    project = etree.SubElement(header, "project")
    project.text = report_parameters.get('project')
    date = etree.SubElement(header, "date")
    date.text = time.strftime("%m/%d/%Y")
    redcapServerAddress = etree.SubElement(header, "redcapServerAddress")
    redcapServerAddress.text = report_parameters.get('redcap_uri')


def updateReportSummary(root, report_data):
    summary = root[1]
    subjectCount = etree.SubElement(summary, "subjectCount")
    subjectCount.text = str(report_data.get('total_subjects'))
    forms = etree.SubElement(summary, "forms")
    form_data = report_data['form_details']
    for k in sorted(form_data.keys()):
        form = etree.SubElement(forms, "form")
        name_element = etree.SubElement(form, "form_name")
        name_element.text = k
        count_element = etree.SubElement(form, "form_count")
        count_element.text = str(form_data.get(k))


def updateReportAlerts(root, alert_summary):
    alerts = root[2]
    too_many_forms = etree.SubElement(alerts, 'tooManyForms')
    too_many_values = etree.SubElement(alerts, 'tooManyValues')
    for event in alert_summary['max_event_alert']:
        event_alert = etree.SubElement(too_many_forms, 'eventAlert')
        msg = etree.SubElement(event_alert, 'message')
        msg.text = event
    for value in alert_summary['multiple_values_alert']:
        values_alert = etree.SubElement(too_many_values, 'valuesAlert')
        msg = etree.SubElement(values_alert, 'message')
        msg.text = value


def updateSubjectDetails(root, subject_details):
    """
    Helper method for #create_summary_report()
    Adds subject information to the xml tree which is later formated
    by `redi/utils/report.xsl` into the html `table#subject_details"`
    """
    subjectsDetails = root[3]
    for key in sorted(subject_details.keys()):
        subject = etree.SubElement(subjectsDetails, "subject")
        details = subject_details.get(key)
        subject.append(gen_ele("redcap_id", key))
        forms = etree.SubElement(subject, "forms")

        for k in sorted(details.keys()):
            if(k.endswith("_Forms")):
                form = etree.SubElement(forms, "form")
                name_element = etree.SubElement(form, "form_name")
                name_element.text = k
                count_element = etree.SubElement(form, "form_count")
                count_element.text = str(details.get(k))
            else:
                element = etree.SubElement(subject, k)
                element.text = str(details.get(k))


def updateReportErrors(root, errors):
    errorsRoot = root[4]
    for error in errors:
        errorElement = etree.SubElement(errorsRoot, "error")
        errorElement.text = str(error)


def updateSummaryOfSpecimenTakenTimes(root, collection_date_summary_dict):
    timeSummaryRoot = root[5]
    totalElement = etree.SubElement(timeSummaryRoot, "total")
    totalElement.text = str(collection_date_summary_dict['total'])
    blankElement = etree.SubElement(timeSummaryRoot, "blank")
    blankElement.text = str(collection_date_summary_dict['blank'])
    percentElement = etree.SubElement(timeSummaryRoot, "percent")
    percentElement.text = str((float(collection_date_summary_dict['blank'])/\
        collection_date_summary_dict['total'])*100)


def gen_ele(ele_name, ele_text):
    """ Create an xml element with given name and content """
    return etree.XML("<{}>{}</{}>".format(ele_name, ele_text, ele_name))


def gen_subele(parent, subele_name, subele_text):
    subele = etree.SubElement(parent, subele_name)
    subele.text = subele_text
    return subele
